@@ -76,9 +76,6 @@ The filename will be appended to the command."] \
 proc Edit_Draft {} {
     global mhProfile exmh
     # Run the editor on the current draft
-    if {$exmh(folder) == $mhProfile(draft-folder)} {
-	Msg_CheckPoint	;# Update cur message
-    }
     set draftID [Mh_Cur $mhProfile(draft-folder)]
     if {$draftID == {}} {
 	Exmh_Status "No current message in Draft-Folder $mhProfile(draft-folder)" error
@@ -474,10 +471,11 @@ proc Edit_Done {act {msg cur}} {
 	    Quote_Cleanup
 	    if {$anno} {
 		if {[string compare $exmh($msg,folder) $exmh(folder)] == 0} {
-		    set ix [Ftoc_FindMsg $exmh($msg,mhmessages)]
-		    Exmh_Debug Edit_Done ix=$ix
-		    if {$ix != {}} {
-			Ftoc_RescanLine $ix dash
+		    foreach lineno [Ftoc_FindMsgs $exmh($msg,mhmessages)] {
+			Exmh_Debug Edit_Done lineno=$lineno
+			if {$lineno != {}} {
+			    Ftoc_RescanLine $lineno dash
+			}
 		    }
 		}
 	    }
