@@ -74,9 +74,9 @@ This can slow down message displaying somewhat,
 so you may prefer to do this manually by typing <Key-z>
 (WWW Preferences.)"}
 
-	{ unseenwin(on) unseenWinOn OFF {Enable Unseen Window}
-"Enables the window that summarizes your unseen messages.
-(Unseen Window Preferences.)" }
+	{ seqwin(on) seqWinOn OFF {Enable Sequences Window}
+"Enables the window that summarizes your sequences.
+(Sequences Window Preferences.)" }
 
 	{ sound(enabled) soundEnabled ON {Sound feedback}
 "Enable audio feedback.  Exmh will make a sound when
@@ -314,9 +314,9 @@ resources for tuning *quoteOmit.
 non-empty, the function in *quote_type is run on that MIME section.
 For instance, if the following resources were set:
 
-*quoteUAdd: message/rfc822 text/* multipart/* application/pgp \
+*uquoteAdd: message/rfc822 text/* multipart/* application/pgp \
     foo1/bar1 foo2/bar2 foo3/*
-*quoteUOmit: foo3/barnone
+*uquoteOmit: foo3/barnone
 *quote_message/rfc822:          Quote_Message
 *quote_multipart/signed:        Quote_MultipartSigned
 *quote_multipart/encrypted:     Quote_MultipartEncrypted
@@ -791,139 +791,3 @@ mail sent to addresses only in these domains will always be signed
 with the local signature."}
     }
 }
-
-proc UnseenWin_Init {} {
-  global unseenwin
-
-  Preferences_Add "Unseen Window" \
-"The unseen window, if enabled, shows a list of folders with unseen
-messages (together with the number of unseen messages).  Options exist
-to control its size (which in turn controls its behaviour as entries
-are inserted or removed and also to control the action that will occur
-if you click in the window." \
-  {
-    { unseenwin(on) unseenWinOn OFF {Enable Unseen Window}
-"Enables the unseen window."
-    }
-
-    { unseenwin(icon) unseenWinIcon OFF {Icon Window}
-"Tries to tell the window manager to use the unseen window as
-exmh's icon.
-Note: doesn't work with all window managers."
-    }
-
-    { unseenwin(minlines) unseenWinMinLines 3 {Minimum Entry Lines}
-"The minimum number of entries that the unseen window will show.
-This controls the minimum height that the unseen window will adopt."
-    }
-
-    { unseenwin(maxlines) unseenWinMaxLines 10 {Maximum Entry Lines}
-"The maximum number of entries that the unseen window will show.
-
-If set and the size of the entries exceeds the maximum,
-scanning (using button 2) is enabled in the window.
-
-If left blank (or set to a value less than the minimum),
-the icon window will always grow to accomodate the current
-number of entries."
-    }
-
-    { unseenwin(minwidth) unseenWinMinNameWidth 11 {Minimum Name Width}
-"The minimum number of characters that will be used
-for the folder name.  This in turn controls the minimum
-width that the unseen window will adopt."
-    }
-
-    { unseenwin(maxwidth) unseenWinMaxNameWidth 20 {Maximum Name Width}
-"The maximum number of characters that will be used
-for the folder name.
-
-If set and a folders name exceeds the width, it is cropped
-to display just the last characters preceeded by ellipsis such
-that it doesn't exceed the given width.
-
-If left blank (or set to a value less than the minimum width),
-the unseen window will always grow to accomodate the maximum
-folder width."
-    }
-
-    { unseenwin(font) unseenWinFont 6x10 {Font}
-"The font to use in the unseen window."
-    }
-
-    { unseenwin(hidewhenempty) unseenWinHideWhenEmpty OFF {Hide When Empty}
-"The unseen window will only be displayed when there are unseen
-messages."
-    }
-
-    { unseenwin(emptymsg) unseenWinEmptyMsg {NO UNREAD MAIL} {Empty Message}
-"The message to display in the unseen window when there are
-no unread messages.
-
-Note; if \"Hide When Empty\" is on, this option has no use."
-    }
-
-    { unseenwin(b1mode) unseenWinButton1
-      {CHOICE None Raise Warp {Warp & Show}}
-      {Button 1 Mode}
-"The behaviour when you click on the unseen window depends
-on this setting.
-
-None:        Nothing
-Raise:       Bring the main window to the front
-             (deiconifying if necessary)
-Warp:        Raise then change to the folder clicked on
-Warp & Show: Warp then select the first unseen message"
-    }
-
-    { unseenwin(b2mode) unseenWinButton2
-      {CHOICE Nothing Inc}
-      {Button 2 Mode}
-"The behaviour when you press mouse button 2 depends on
-this setting.
-
-Nothing:     Does nothing
-Inc:         Just like clicking the Inc button"
-    }
-
-    { unseenwin(mb1mode) unseenWinModifiedButton1
-      {CHOICE None Raise Warp {Warp & Show}}
-      {Modified Button 1 Mode}
-"The behaviour when you shift-click, control-click or
-shift-contol-click on the unseen window depends
-on this setting.
-
-None:        Nothing
-Raise:       Bring the main window to the front
-             (deiconifying if necessary)
-Warp:        Raise then change to the folder clicked on
-Warp & Show: Warp then select the first unseen message"
-    }
-
-    { unseenwin(b3mode) unseenWinButton3
-      {CHOICE Nothing Compose}
-      {Button 3 Mode}
-"The behaviour when you press mouse button 3 depends on
-this setting.
-
-Nothing:     Does nothing
-Compose:     Starts a mail composition"
-    }
-  }
-
-  set unseenwin(digits) 1
-  trace variable unseenwin(on)            w UnseenWinToggle
-  trace variable unseenwin(icon)          w UnseenWinToggleIcon
-  trace variable unseenwin(font)          w UnseenWinChangeFont
-  trace variable unseenwin(emptymsg)      w UnseenWinEmptyMsg
-  trace variable unseenwin(hidewhenempty) w UnseenWinEmptyMsg
-  foreach v {minlines maxlines minwidth maxwidth} {
-    trace variable unseenwin($v) w UnseenWinChangeMinMax
-  }
-  foreach v {b1mode mb1mode} {
-    trace variable unseenwin($v) w UnseenWinToggleClick
-  }
-  if {$unseenwin(on)} {
-      UnseenWinToggle
-  }
-}
