@@ -143,13 +143,12 @@ proc Exwin_Layout {} {
     set exwin(ftext) [Widget_Text [Widget_Frame . ftoc Ftoc $fixed] \
 				$exwin(ftextLines)]
     Ftoc_Bindings $exwin(ftext)
+    Ftoc_InitSequences $exwin(ftext)
 
     if {$exwin(wheelEnabled)} {
 	mscroll $exwin(ftext) 1
     }
 
-    Ftoc_ColorConfigure $exwin(ftext)
-
     # Frame for faces, status, message buttons
     set mid     [Widget_Frame . mid Mid $fixed]
     Widget_SplitFrameR $mid Face Right
@@ -434,6 +433,19 @@ proc Exwin_Toplevel { path name {class Dialog} {dismiss yes}} {
 		Exmh_Debug Exwin_Toplevel $path $exwin(geometry,$path)
 	    }
 	    wm deiconify $path
+
+            # Some window managers (KDE 3.0?) need extra coaxing
+            # to get the dialog to appear on top
+
+            # This update make windows appear before they
+            # update
+            # are fully populated, or, for sedit, they appear with
+            # old data and then are redrawn.  I'm running on a slow 
+            # displayand it is annoying
+
+            # But this raise reportedly helps
+            raise $path
+
 	} else {
 	    catch {
 		if {! $exwin(keepPlaces)} {
