@@ -271,7 +271,7 @@ proc FacePathlist { from } {
     for {set i [expr [llength $from]-1]} {$i>=0} {incr i -1} {
 	append path $prefix [lindex $from $i]
 	set prefix /
-	set pathlist [concat $path $pathlist]
+	set pathlist [linsert $pathlist 0 $path]
     }
     lappend pathlist {}
     return $pathlist
@@ -426,7 +426,7 @@ proc FaceShowFile {facefile pane} {
 
 proc FaceXFace { xface {pane {}}} {
     global faces
-    Exmh_Status "$faces(xfaceProg)" red
+    Exmh_Status "$faces(xfaceProg)" warning
 # Tputs decode x-face: [time {
     if [catch {open "| $faces(xfaceProg) > [Env_Tmp]/FACE.[pid].xbm" w} fid] {
 	Exmh_Status $fid error
