@@ -31,35 +31,44 @@ proc Flist_Init {} {
     Flist_FindAllFolders
 }
 proc FlistResetVars {} {
-    global flist
-    set flist(unseen) {}	;# Sequence of folders to visit
+    global flist mhProfile
+    set flist($mhProfile(unseen-sequence)) {}	;# Sequence of folders to visit
     set flist(unvisited) {}	;# Unseen folders not yet visited
     Exmh_Debug FlistResetVars
     set flist(unvisitedNext) {}	;# Temporary copy (next iteration)
-    set flist(newMsgs) 0	;# Total count of unseen messages
 
-    # flist(new,$folder)	is the number of unseen in a folder
-    #				(Update with care; 'trace'd by UnseenWinTrace)
-    # flist(newseq,$folder)	is the message id's of unseen messages
-    # flist(mtime,$folder)	is modify time of .mh_sequences file
-    foreach x [array names flist] {
-	if [regexp {^(new|newseq|mtime),} $x] {
-	    # reset per-folder new and newseq state
-	    unset flist($x)
+    # flist(seqcount,$folder,$seq)
+    #	number of sequence elements in a folder
+    #	(Update with care; 'trace'd by Seq_Trace)
+    # flist(oldseqcount,$folder,$seq)
+    #	previous number of sequence elements in a folder
+    # flist(seq,$folder,$seq)
+    #	message id's of messages in $seq
+    # flist(mtime,$folder)
+    #	modification time of .mh_sequences file
+    # flist(totalcount,$seq)	;# Total count of messages in sequence
+
+    foreach x [lsort -ascii [array names flist]] {
+	if [regexp {^(seq),} $x] {
+	    # reset state
+	    set flist($x) {}
+	} elseif [regexp {^(mtime|totalcount|seqcount|oldseqcount),} $x] {
+	    # reset state
+	    set flist($x) 0
 	}
     }
     if ![info exists flist(debug)] {
 	set flist(debug) 0
     }
     if {$flist(debug)} {
-	trace variable flist(newMsgs) w FlistTraceNewMsgs
-	trace variable flist(unseen) w FlistTraceUnseen
+	trace variable flist(totalcount,$mhProfile(unseen-sequence)) w FlistTraceTotalUnseen
+	trace variable flist($mhProfile(unseen-sequence)) w FlistTraceUnseen
 	trace variable flist(unvisited) w FlistTraceUnvisited
 	set flist(listbox) .flistbox
 	set f $flist(listbox)
 	if ![winfo exists $f] {
 	    Exwin_Toplevel $f "Flist Debug" Flist
-	    Widget_Label $f newMsgs {top fillx } -textvariable flist(newMsgs)
+	    Widget_Label $f newMsgs {top fillx } -textvariable flist(totalcount,$mhProfile(unseen-sequence))
 	    Widget_Frame $f top Labels
 	    Widget_Label $f.top unseen {left fill expand} -text Unseen
 	    Widget_Label $f.top unvisited {left fill expand} -text Unvisited
@@ -69,16 +78,16 @@ proc FlistResetVars {} {
 	}
     }
 }
-proc FlistTraceNewMsgs {args} {
-    global flist
+proc FlistTraceTotalUnseen {args} {
+    global flist mhProfile
     set l [info level]
     incr l -1
-    Exmh_Debug flist(newMsgs) => $flist(newMsgs) : [info level $l] $args
+    Exmh_Debug flist(totalcount,$mhProfile(unseen-sequence)) => $flist(totalcount,$mhProfile(unseen-sequence)) : [info level $l] $args
 }
 proc FlistTraceUnseen {args} {
     global flist
     $flist(listbox).unseen delete 0 end
-    foreach f $flist(unseen) {
+    foreach f $flist($mhProfile(unseen-sequence)) {
 	$flist(listbox).unseen insert end $f
     }
 }
@@ -100,7 +109,7 @@ proc Flist_Refresh {} {
     FlistResetVars
     FlistFindAllInner
     Fdisp_Redisplay
-    Flist_FindUnseen 1
+    Flist_FindSeqs 1
     Folder_FindShared
     Inc_PresortFinish
 }
@@ -195,7 +204,7 @@ proc Flist_DelFolder { folder } {
 proc FlistCacheFolderList {} {
     global flist
     if [catch {open $flist(cacheFile) w} out] {
-	Exmh_Status "Cannot cache folder list: $out" red
+	Exmh_Status "Cannot cache folder list: $out" warning
     } else {
 	foreach f $flist(allfolders) {
 	    puts $out $f
@@ -256,143 +265,10 @@ proc Flist_FolderSet { {subfolder .} } {
 	return $result
     }
 }
-
-# The routines below here manage the unseen sequence state per folder.
-
-proc FlistSeq { folder sequence } {
-    global mhProfile
-    # Explode a sequence into a list of message numbers
-    set seq {}
-    set rseq {}
-    foreach range [split [string trim $sequence]] {
-	set parts [split [string trim $range] -]
-	if {[llength $parts] == 1} {
-	    lappend seq $parts
-	    set rseq [concat $parts $rseq]
-	} else {
-	    for {set m [lindex $parts 0]} {$m <= [lindex $parts 1]} {incr m} {
-		lappend seq $m
-		set rseq [concat $m $rseq]
-	    }
-	}
-    }
-    # Hack to weed out unseen sequence numbers for messages that don't exist
-    foreach m $rseq {
-	if ![file exists $mhProfile(path)/$folder/$m] {
-	    Exmh_Debug $mhProfile(path)/$folder/$m not found
-	    set ix [lsearch $seq $m]
-	    set seq [lreplace $seq $ix $ix]
-	} else {
-	    # Real hack
-	    break
-	}
-    }
-    return $seq
-}
-
-# Reset the cached state about unseen messages because the user
-# has just packed, sorted, or threaded the folder.
-# This should be followed shortly by a call to Flist_UnseenMsgs
-#
-# Don't call gratitiously because it confuses the exmhunseen window.
-
-proc Flist_ForgetUnseen {folder} {
-    global flist
-    set flist(newseq,$folder) {}
-    if {[info exists flist(new,$folder)]} {
-	set flist(newMsgs) [expr $flist(newMsgs) - $flist(new,$folder)]
-    }
-    set flist(new,$folder) 0
-    set ix [lsearch $flist(unseen) $folder]
-    if {$ix >= 0} {
-	set flist(unseen) [lreplace $flist(unseen) $ix $ix]
-    }
-}
-
-# Add unseen messages to the list for a given folder.
-# This has to be careful about already known unseen messages
-# and messages that have been read but not committed as read.
-proc Flist_AddUnseen {folder seq} {
-    global flist exmh
-
-    # Check overlap with already seen msgs and unseen messages already known
-    if [info exists flist(newseq,$folder)] {
-	if [info exists flist(new,$folder)] {
-	    set new $flist(new,$folder)
-	} else {
-	    set new 0
-	}
-    } else {
-	set flist(newseq,$folder) {}
-	set new 0
-    }
-    if {[string compare $folder $exmh(folder)] == 0} {
-	set known [concat [Msg_Seen] $flist(newseq,$folder)]
-    } else {
-	set known $flist(newseq,$folder)
-    }
-    # Subtract elements of $known from $seq
-    if {[llength $known] > [llength $seq]} {
-	set nseq {}
-	foreach id $seq {
-	    if {[lsearch $known $id] < 0} {
-		lappend nseq $id
-	    }
-	}
-	set seq $nseq
-    } else {
-	foreach id $known  {
-	    set ix [lsearch $seq $id]
-	    if {$ix >= 0} {
-		set seq [lreplace $seq $ix $ix]
-	    }
-	}
-    }
-    set num [llength $seq]
-    if {$num <= 0} {
-	return
-    }
-    incr flist(newMsgs) $num
-    set flist(new,$folder) [expr $new + $num]
-    set flist(newseq,$folder) [concat $flist(newseq,$folder) $seq]
-    if {[lsearch $flist(unseen) $folder] < 0} {
-	lappend flist(unseen) $folder
-    }
-    if {[string compare $folder $exmh(folder)] != 0 &&
-	[lsearch $flist(unvisited) $folder] < 0} {
-	lappend flist(unvisitedNext) $folder
-    }
-    Fdisp_HighlightUnseen $folder
-    if 0 {
-	# This line of code causes doubling of messages in the scan listing.
-	# don't do it
-	if {[string compare $exmh(folder) $folder] == 0} {
-	  Scan_Folder $folder
-	}
-    }
-}
 proc Flist_Done {} {
     global flist exmh
 
     Exmh_Debug Flist_Done
-    if {$flist(newMsgs) > 0} {
-	if {$flist(newMsgs) == 1} {set msg "msg"} else {set msg "msgs"}
-	if {[llength $flist(unseen)] == 1} {set f "folder"} else {set f "folders"}
-	Exmh_Status "$flist(newMsgs) unread $msg in [llength $flist(unseen)] $f" blue
-	if ![info exists flist(lastNewMsgs)] {
-	    set flist(lastNewMsgs) 0
-	}
-	set delta [expr {$flist(newMsgs) - $flist(lastNewMsgs)}]
-	if {$delta > 0} {
-	    Flag_NewMail
-	    Sound_Feedback $delta
-	}
-    } else {
-	set flist(newMsgs) 0
-	Flag_NoUnseen
-	Exmh_Status "No unread msgs" blue
-    }
-    set flist(lastNewMsgs) $flist(newMsgs)
     set flist(unvisited) [FlistSort $flist(unvisitedNext)]
     set flist(active) 0
 }
@@ -402,48 +278,34 @@ proc Flist_Done {} {
 # they add messages to a folder
 
 proc Flist_UnseenUpdate { folder } {
-    global exmh flist
-    Flist_ResetUnseen $folder
-    Flist_UnseenMsgs $folder
-    if {[string compare $folder $exmh(folder)] == 0} {
-	Scan_FolderUpdate $folder
+    global exmh flist ftoc mhProfile
+    Exmh_Debug Flist_UnseenUpdate $folder
+    foreach seq [Mh_Sequences $folder] {
+	Seq_Set $folder $seq [MhGetSeqCache $folder $seq]
     }
-    if {[lsearch $flist(unvisited) $folder] < 0} {
+    if {[string compare $folder $exmh(folder)] == 0} {
+	if {$ftoc(autoSort)} {
+	    if [Flist_NumUnseen $folder $mhProfile(unseen-sequence)] {
+		Ftoc_Sort
+	    }
+	}
+	Scan_FolderUpdate $folder
+    } elseif {[lsearch $flist(unvisited) $folder] < 0} {
 	lappend flist(unvisited) $folder
 	set flist(unvisitedNext) $flist(unvisited)
     }
     # This wiggles the flag and sorts flist(unvisited)
     Flist_Done
 }
-proc Flist_ResetUnseen { folder } {
-    global flist
-    if [Flist_NumUnseen $folder] {
-	incr flist(newMsgs) -$flist(new,$folder)
-	set flist(newseq,$folder) {}
-	set flist(new,$folder) 0
-    }
-}
-proc Flist_UnseenMsgs { folder } {
-    global flist
-    Flist_AddUnseen $folder [Mh_Unseen $folder]
-    return $flist(newseq,$folder)
-}
-proc Flist_NumUnseen { folder } {
-    global flist
-    if [info exists flist(new,$folder)] {
-	return $flist(new,$folder)
-    } else {
-	return 0
-    }
-}
 proc Flist_UnseenFolders {} {
-    global flist
-    return $flist(unseen)
+    global flist mhProfile
+    return $flist($mhProfile(unseen-sequence))
 }
 
 # Flist enumerates folders that have unseen messages.
-proc Flist_FindUnseen {{reset 0}} {
-    Exmh_Debug Flist_FindUnseen end [time [list FlistFindUnseen $reset]]
+proc Flist_FindSeqs {{reset 0}} {
+    Exmh_Debug Flist_FindSeqs reset=$reset
+    Exmh_Debug Flist_FindSeqs end [time [list FlistFindSeqs $reset]]
 }
 
 proc FlistFindStart {reset} {
@@ -462,55 +324,34 @@ proc FlistFindStart {reset} {
     }
     return 1
 }
-proc FlistFindUnseen {reset} {
-    global mhProfile flist curFolder
-    Exmh_Debug FlistFindUnseen reset=$reset
-    if {![BgRPC FlistFindStart $reset]} {
-	# Flist active
-	return
-    }
+
+proc FlistFindSeqsInner {} {
+    global flist seqwin
     if {[catch {
-	FlistGetContext
-	set result {}
-	set keep {}
-	foreach f $flist(unseenfolders) {
-	    set hit 0
-	    foreach line $flist(context) {
-		set line [split $line]
-		set key [lindex $line 0]
-		if {$key == "atr-$mhProfile(unseen-sequence)-$mhProfile(path)/$f:"} {
-		    set seq [lindex [split $line :] 1]
-		    BgRPC Flist_AddUnseen $f [FlistSeq $f $seq]
-		    set hit 1
-		    break
-		}
-	    }
-	    if {! $hit} {
-		set path $mhProfile(path)/$f/$mhProfile(mh-sequences)
-		if {![file exists $path] ||
-		    ([info exists flist(mtime,$f)] &&
-		     ([file mtime $path] <= $flist(mtime,$f)))} {
-		    # No state to report
-		} else {
-		    if {[catch {open $path} in] == 0} {
-			set se \n[read $in]
-			if [regexp \n$mhProfile(unseen-sequence):\[^\n\]*\n $se line] {
-			    set seq [lindex [split $line :\n] 2]
-			    BgRPC Flist_AddUnseen $f [FlistSeq $f $seq]
-			}
-			close $in
-			set flist(mtime,$f) [file mtime $path]
-		    }
-		}
-	    }
+    FlistGetContext
+    foreach folder $flist(unseenfolders) {
+        foreach seq [Mh_Sequences $folder] {
+            if {[lsearch $seqwin(nevershow) $seq] < 0} {
+                Seq_Set $folder $seq [MhGetSeqCache $folder $seq]
+            }
 	}
+    }
     } err]} {
 	# An error here is most likely a flakey NFS connection
 	# It is important to trap this so we can mark the
 	# flist action as "Done" below.  Otherwise, we'll stop
 	# looking for new messages.
-	Exmh_Debug "FlistFindUnseen: $err"
+	Exmh_Debug "FlistFindSeqs: $err"
     }
+}
+
+proc FlistFindSeqs {reset} {
+    global flist
+    if {![BgRPC FlistFindStart $reset]} {
+	# Flist active
+	return
+    }
+    BgRPC FlistFindSeqsInner
     BgRPC Flist_Done
 }
 proc FlistGetContext {} {
@@ -523,39 +364,19 @@ proc FlistGetContext {} {
 	}
     }
 }
-proc Flist_MsgSeen { msgid } {
-    global flist exmh
-    if [info exists flist(newseq,$exmh(folder))] {
-	set ix [lsearch $flist(newseq,$exmh(folder)) $msgid]
-	if {$ix >= 0} {
-	    set flist(newseq,$exmh(folder)) \
-		[lreplace $flist(newseq,$exmh(folder)) $ix $ix]
-	    incr flist(new,$exmh(folder)) -1
-	    incr flist(newMsgs) -1
-	    set flist(lastNewMsgs) $flist(newMsgs)
-	    if {$flist(new,$exmh(folder)) == 0} {
-		FlistUnseenFolder $exmh(folder)
-	    }
-	    if {$flist(newMsgs) <  0} {
-		Exmh_Status "$flist(newMsgs) unseen!"
-		set flist(newMsgs) 0
-	    }
-	}
-    }
-}
 proc Flist_SeenAll { folder } {
     FlistUnseenFolder $folder
 }
 proc FlistUnseenFolder { folder } {
-    global flist
+    global flist mhProfile
     Exmh_Debug FlistUnseenFolder $folder
-    catch {unset flist(new,$folder)}
-    catch {unset flist(newseq,$folder)}
+    set flist(seqcount,$folder,$mhProfile(unseen-sequence)) 0
+    set flist(seq,$folder,$mhProfile(unseen-sequence)) {}
     Fdisp_UnHighlightUnseen $folder
-    set ix [lsearch $flist(unseen) $folder]
+    set ix [lsearch $flist($mhProfile(unseen-sequence)) $folder]
     if {$ix >= 0} {
-	set flist(unseen) [lreplace $flist(unseen) $ix $ix]
-	if {[llength $flist(unseen)] == 0} {
+	set flist($mhProfile(unseen-sequence)) [lreplace $flist($mhProfile(unseen-sequence)) $ix $ix]
+	if {[llength $flist($mhProfile(unseen-sequence))] == 0} {
 	    Flag_NoUnseen
 	}
     }
@@ -633,16 +454,16 @@ set flist(cacheFile) /usr/joe/Mail/.exmhfolders
     return $result
 }
 
-proc Flist_NextUnseen { } {
+proc Flist_NextUnvisited { } {
     # Return the next folder in Folder-Order that has unseen messages
-    global flist exmh
+    global flist exmh mhProfile
 
     foreach f $flist(unvisited) {
 	if {[string compare $f $exmh(folder)] != 0} {
 	    return $f
 	}
     }
-    foreach f $flist(unseen) {
+    foreach f $flist($mhProfile(unseen-sequence)) {
 	if {[string compare $f $exmh(folder)] != 0} {
 	    return $f
 	}
