@@ -20,10 +20,10 @@ proc Folder_Init {} {
 	#scan named folder
 	set exmh(folder) $argv
     } else {
-	if [catch {exec folder -fast < /dev/null} f] {
+	if [catch {exec folder -fast < /dev/null} folder] {
 	    set exmh(folder) {}
 	} else {
-	    set exmh(folder) $f
+	    set exmh(folder) $folder
 	}
     }
 }
@@ -63,13 +63,12 @@ proc Folder_Summary { folder } {
     }
 }
 
-proc Folder_Change {f {msgShowProc Msg_ShowCurrent}} {
-#    LogStart "Folder_Change $f"
-    Exmh_Debug ****************
-    Exmh_Debug Folder_Change $f [time [list  FolderChange $f $msgShowProc]]
+proc Folder_Change {folder {msgShowProc {Msg_Show cur}}} {
+    Exmh_Debug Folder_Change $folder $msgShowProc
+    Exmh_Debug Folder_Change $folder [time [list  FolderChange $folder $msgShowProc]]
 }
-proc FolderChange {f msgShowProc} {
-    global exmh mhProfile ftoc
+proc FolderChange {folder msgShowProc} {
+    global exmh mhProfile ftoc msg
     if {[string compare [wm state .] normal] != 0} {
 	if {$exmh(iconic)} {
 	    # Ignore once if starting up with -iconic flag
@@ -83,22 +82,23 @@ proc FolderChange {f msgShowProc} {
 	return
     }
     # Trim off leading mail path
-    if [regsub ^$mhProfile(path)/ $f {} newf] {
-	set f $newf
+    if [regsub ^$mhProfile(path)/ $folder {} newfolder] {
+	set folder $newfolder
     }
-    if {[string length $f] == 0} {
+    if {[string length $folder] == 0} {
 	return
     }
-    if ![file isdirectory $mhProfile(path)/$f] {
-	Exmh_Status "Folder $f doesn't exist" purple
+    if ![file isdirectory $mhProfile(path)/$folder] {
+	Exmh_Status "Folder $folder doesn't exist" error
 	return
     }
     set oldFolder $exmh(folder)
-    Exmh_Status "Changing to $f ..."
-    if {$f != $exmh(folder)} {
+    Exmh_Status "Changing to $folder ..."
+    set msg(id) ""
+    if {$folder != $exmh(folder)} {
 	Exmh_Debug Exmh_CheckPoint [time Exmh_CheckPoint]
 	global mhProfile
-	set summary [Mh_Folder $f]	;# Set MH folder state
+	set summary [Mh_Folder $folder]	;# Set MH folder state
     } else {
 	if {$ftoc(folder) == {} && $exmh(started)} {
 	    # pseudo-display -> Checkpoint to set cur msg
@@ -107,66 +107,62 @@ proc FolderChange {f msgShowProc} {
         }
 	set summary {}
     }
+    set ftoc(displayValid) 1
     set exmh(started) 1
     global folderHook
     if [info exists folderHook(leave,$oldFolder)] {
 	$folderHook(leave,$oldFolder) $oldFolder leave
     }
-    Label_Folder $f $summary
-    Fdisp_HighlightCur $f
-    Flist_Visited $f
-    set exmh(folder) $f
-    if {$ftoc(autoSort)} {
-	if [Flist_NumUnseen $f] {
-	    Ftoc_Sort
-	}
-    }
-    Scan_Folder $f 1
-    Exmh_Status $f
-    # Either Msg_ShowCurrent or Msg_ShowUnseen
+    Label_Folder $folder $summary
+    Fdisp_HighlightCur $folder
+    Flist_Visited $folder
+    set exmh(folder) $folder
+    Flist_UnseenUpdate $folder
+    Scan_CacheUpdate
+    Ftoc_ShowSequences
+    Exmh_Status $folder
+    # Usually {Msg_Show $seq} or {Msg_Change $msg}
     eval $msgShowProc
 
     # Take any required folder-specific action (e.g., for drafts folder)
-    if [info exists folderHook(enter,$f)] {
-	$folderHook(enter,$f) $f enter
+    if [info exists folderHook(enter,$folder)] {
+	$folderHook(enter,$folder) $folder enter
     }
     foreach cmd [info commands Hook_FolderChange*] {
-	$cmd $f
+	$cmd $folder
+    }
+    foreach seq [option get . sequences {}] {
+	BgRPC Seq_Msgs $oldFolder $seq
     }
 }
 
-proc Folder_Unseen {} {
-    Ftoc_NextFolder
-#    Folder_Change [Flist_NextUnseen]
-}
-
-proc Folder_Target {f} {
+proc Folder_Target {folder} {
     global exmh mhProfile
 
-    if ![file isdirectory $mhProfile(path)/$f] {
-	Exmh_Status "$mhProfile(path)/$f doesn't exist"
+    if ![file isdirectory $mhProfile(path)/$folder] {
+	Exmh_Status "$mhProfile(path)/$folder doesn't exist"
 	return 0
     }
-    if {$exmh(folder) == $f} {
-	Exmh_Status "Target must be different than current" red
+    if {$exmh(folder) == $folder} {
+	Exmh_Status "Target must be different than current" warning
 	return 0
     }
-    Fdisp_HighlightTarget $f
-    set exmh(target) $f
-    Exmh_Status "$f is target for moves and copies"
+    Fdisp_HighlightTarget $folder
+    set exmh(target) $folder
+    Exmh_Status "$folder is target for moves and copies"
     return 1
 }
-proc Folder_TargetMove { f {moveProc Ftoc_MoveMark} } {
-    if [Folder_Target $f] {
+proc Folder_TargetMove { folder {moveProc Ftoc_MoveMark} } {
+    if [Folder_Target $folder] {
 	Msg_Move $moveProc
-	Fcache_Folder $f
+	Fcache_Folder $folder
     }
 }
 
-proc Folder_TargetCopy { f {copyProc Ftoc_CopyMark} } {
-    if [Folder_Target $f] {
+proc Folder_TargetCopy { folder {copyProc Ftoc_CopyMark} } {
+    if [Folder_Target $folder] {
 	Msg_Move $copyProc advance?
-	Fcache_Folder $f
+	Fcache_Folder $folder
     }
 }
 
@@ -184,13 +180,12 @@ proc Folder_Sort { args } {
 
     if {[Ftoc_Changes "Sort"] == 0} then {
 	Background_Wait
-	Exmh_Status "Sorting folder..." blue
+	Exmh_Status "Sorting folder..."
 	eval {Mh_Sort $exmh(folder)} $args
- 	Flist_ResetUnseen $exmh(folder)	 ;# unseen sequence might have changed
 	Scan_FolderForce
 	set id [Mh_Cur $exmh(folder)]
 	if {$id != {}} {
-	    Msg_Change $id
+	    Msg_Change $id skipdisplay
 	} else {
 	    Msg_ClearCurrent
 	}
@@ -198,9 +193,9 @@ proc Folder_Sort { args } {
 }
 
 proc Folder_Previous {} {
-    set f [Ftoc_LastFolder]
-    if {[string length $f]} {
-	Folder_Change $f
+    set folder [Ftoc_LastFolder]
+    if {[string length $folder]} {
+	Folder_Change $folder
     }
 }
 
@@ -209,13 +204,12 @@ proc Folder_Pack {} {
 
     if {[Ftoc_Changes "Pack"] == 0} then {
 	Background_Wait
-	Exmh_Status "Packing folder..." blue
+	Exmh_Status "Packing folder..."
 	Mh_Pack $exmh(folder)
- 	Flist_ResetUnseen $exmh(folder)	 ;# unseen sequence might have changed
 	Scan_FolderForce
 	set id [Mh_Cur $exmh(folder)]
 	if {$id != {}} {
-	    Msg_Change $id
+	    Msg_Change $id skipdisplay
 	} else {
 	    Msg_ClearCurrent
 	}
@@ -228,7 +222,6 @@ proc Folder_Commit { {rmmCommit Mh_Rmm} {moveCommit Mh_Refile} {copyCommit Mh_Co
 proc FolderCommit { rmmCommit moveCommit copyCommit } {
     global exmh exwin ftoc
 
-    Msg_CheckPoint	;# Update sequence state
     Ftoc_Commit $rmmCommit $moveCommit $copyCommit
     Exmh_Debug Scan_CacheUpdate [time Scan_CacheUpdate]
 
@@ -243,7 +236,6 @@ proc Folder_CommitType { type } {
     global ftoc exmh
     Exmh_Debug Folder_CommitType $type
     if {[string compare $type "Change folder"] == 0} {
-	Msg_CheckPoint
 	busy Ftoc_Commit Mh_Rmm Mh_Refile Mh_Copy
 	if $ftoc(autoPack) {
 	    Background_Wait	;# Let folder ops complete
@@ -276,10 +268,10 @@ proc Folder_Purge { {folder {}} } {
     }
     set purgesecs [expr $mhProfile(purgeage) * 24 * 60 * 60]
     set n 0
-    foreach f [glob -nocomplain $mhProfile(path)/$folder/$mhProfile(delprefix)*] {
-	if {[file mtime $f] + $purgesecs < $now} {
-	    Exmh_Debug Purge $f
-	    File_Delete $f
+    foreach file [glob -nocomplain $mhProfile(path)/$folder/$mhProfile(delprefix)*] {
+	if {[file mtime $file] + $purgesecs < $now} {
+	    Exmh_Debug Purge $file
+	    File_Delete $file
 	    incr n
 	}
     }
@@ -292,8 +284,8 @@ proc Folder_Purge { {folder {}} } {
 proc Folder_PurgeAll {} {
     global flist
     set n 0
-    foreach f $flist(allfolders) {
-	incr n [Folder_Purge $f]
+    foreach folder $flist(allfolders) {
+	incr n [Folder_Purge $folder]
     }
     if {$n > 0} {
 	Exmh_Status "Folder_PurgeAll $n msgs purged total"
@@ -332,16 +324,6 @@ proc Folder_PurgeAllBg {} {
     global flist
     Folder_PurgeBg $flist(allfolders)
 }
-
-# Called when changing messages.  If you are sharing a folder,
-# you need to checkpoint state to the file system at each message.
-
-proc Folder_CheckPointShared {} {
-    global exmh folder
-    if {[info exist folder(shared,$exmh(folder))]} {
-	Msg_CheckPoint
-    }
-}
 proc Folder_IsShared {folder} {
     global folder
     set folder(shared,$folder) 1
@@ -352,8 +334,8 @@ proc Folder_FindShared {} {
 	Exmh_Debug $in
 	return
     }
-    foreach f [split [read $in] \n] {
-	Foldler_IsShared $f
+    foreach folder [split [read $in] \n] {
+	Folder_IsShared $folder
     }
     close $in
 }
