@@ -13,7 +13,7 @@
 
 proc Ftoc_Init {} {
     global ftoc
-    set ftoc(displayValid) 0		;# 1 => pick results, not full scan
+    set ftoc(displayValid) 1		;# 0 => pick results, not full scan
     set ftoc(displayDirty) 0		;# 1 => display differs from cache
     set ftoc(mono) [expr {[winfo depth .] <= 4}]
     # Parameters to the Next button
@@ -79,16 +79,22 @@ proc Ftoc_Reset { numMsgs msgid folder } {
     set ftoc(direction) next		;# assumed next direction
     set ftoc(softChange) [expr {! $ftoc(nextGuard)}]
     set ftoc(lasthit) {}		;# search anchor
-    if {$msgid == {}} {
-	set ftoc(curLine) {}		;# current display line number
-    } else {
-	set ftoc(curLine) {}		;# Set later in Msg_Change ?
-    }
+    set ftoc(curLine) $msgid		;# current display line number
+    Ftoc_ClearMsgCache
 }
-proc Ftoc_Update { numMsgs folder } {
+proc Ftoc_Update { numMsgs } {
     # Update size of message list after inc'ing into current folder
     global ftoc
-    Exmh_Debug Ftoc_Update $folder has $numMsgs msgs
+    Exmh_Debug Ftoc_Update $ftoc(folder) has $numMsgs msgs (was $ftoc(numMsgs))
+    if {$numMsgs > $ftoc(numMsgs)} {
+	set msgids {}
+	set lineno $ftoc(numMsgs)
+	while {$lineno < $numMsgs} {
+	    incr lineno
+	    lappend msgids [Ftoc_MsgNumber $lineno]
+	}
+	Ftoc_ShowSequences $msgids
+    }
     set ftoc(numMsgs) $numMsgs
 }
 
@@ -133,26 +139,26 @@ proc Ftoc_Bindings { w } {
 
     Drag_Attach $w FtocDragSelect Shift 3
 }
-proc FtocRangeStart { line } {
+proc FtocRangeStart { lineno } {
     # For normal button-down "start a selection"
     global ftoc
     Ftoc_RangeUnHighlight
-    set ftoc(pickstart) $line
-    set ftoc(pickend) $line
+    set ftoc(pickstart) $lineno
+    set ftoc(pickend) $lineno
     set ftoc(pickstate) new
     set ftoc(extend) 0
-    Ftoc_RangeHighlight $line $line
+    Ftoc_RangeHighlight $lineno $lineno
 }
-proc FtocRangeAdd { line } {
+proc FtocRangeAdd { lineno } {
     # For shift-select "add to selection"
     global ftoc
-    set ftoc(pickstart) $line
-    set ftoc(pickend) $line
+    set ftoc(pickstart) $lineno
+    set ftoc(pickend) $lineno
     set ftoc(pickstate) invert
     set ftoc(extend) 0
-    FtocRangeInvert $line $line
+    FtocRangeInvert $lineno $lineno
 }
-proc FtocRangeEnd { {line {}} {addcurrent 0} } {
+proc FtocRangeEnd { {lineno {}} {addcurrent 0} } {
     # For end of button sweep
     global ftoc exwin
     catch {unset ftoc(extend)}
@@ -160,34 +166,23 @@ proc FtocRangeEnd { {line {}} {addcurrent 0} } {
 	# Spurious button-release event
 	return
     }
-    if {($line == $ftoc(pickstart)) && !$addcurrent} {
+    if {($lineno == $ftoc(pickstart)) && !$addcurrent} {
 	# regular button click optimization
 	unset ftoc(pickend)
-	Msg_Pick $line show
+	Msg_Pick $lineno show
 	return
     }
-    if {$line != {}} {
-	FtocRangeExtend $line
+    if {$lineno != {}} {
+	FtocRangeExtend $lineno
     }
     FtocPickRange $addcurrent
     catch {unset ftoc(pickend)}
 }
-proc Ftoc_PickMsgs { ids addtosel } {
+proc Ftoc_PickMsgs { msgids addtosel } {
     # For adding to the selection by message number
     global ftoc
-    Exmh_Status "Marking [llength $ids] hits"
-    set lines {}
-    for {set L 0} {$L <= $ftoc(numMsgs)} {incr L} {
-	set ix [lsearch $ids [Ftoc_MsgNumber $L]]
-	if {$ix >= 0} {
-	    lappend lines $L
-	    set ids [lreplace $ids $ix $ix]
-	    if {[string length $ids] == 0} {
-		break
-	    }
-	}
-    }
-    Ftoc_LinesHighlight $lines
+    Exmh_Status "Marking [llength $msgids] hits"
+    Ftoc_LinesHighlight [Ftoc_FindMsgs $msgids]
     FtocPickRange $addtosel
 }
 proc FtocRangeExtendXY { x y } {
@@ -237,99 +232,99 @@ proc FtocSelExtend {} {
 	after $widgetText(selectDelay) [list FtocSelExtend]
     }
 }
-proc FtocRangeExtend { line } {
+proc FtocRangeExtend { lineno } {
     global ftoc
     if ![info exists ftoc(pickend)] {
 	return
     }
-    if {$line <= 0} {
-	set line 1
+    if {$lineno <= 0} {
+	set lineno 1
     }
-    if {$line > $ftoc(numMsgs)} {
-	set line $ftoc(numMsgs)
+    if {$lineno > $ftoc(numMsgs)} {
+	set lineno $ftoc(numMsgs)
     }
-    if {$line == $ftoc(pickend)} {
+    if {$lineno == $ftoc(pickend)} {
 	# Invariant, previously defined selection is fine.
 	return
     }
-    if {$line == 0} {
+    if {$lineno == 0} {
 	# no messages in folder
 	return
     }
     if {$ftoc(pickstate) != "invert"} {
 	if {$ftoc(pickstart) < $ftoc(pickend)} {
 	    # Growing downward
-	    if {$line > $ftoc(pickend)} {
-		Ftoc_RangeHighlight [expr $ftoc(pickend)+1] $line
+	    if {$lineno > $ftoc(pickend)} {
+		Ftoc_RangeHighlight [expr $ftoc(pickend)+1] $lineno
 	    } else {
-		if {$line < $ftoc(pickstart)} {
+		if {$lineno < $ftoc(pickstart)} {
 		    if {$ftoc(pickstart) != $ftoc(pickend)} {
 			# Change direction
 			FtocRangeClear [expr $ftoc(pickstart)+1] \
 					$ftoc(pickend)
 		    }
-		    Ftoc_RangeHighlight [expr $ftoc(pickstart)-1] $line
+		    Ftoc_RangeHighlight [expr $ftoc(pickstart)-1] $lineno
 		} else {
 		    # Shrink selection
-		    FtocRangeClear [expr $line+1] $ftoc(pickend)
+		    FtocRangeClear [expr $lineno+1] $ftoc(pickend)
 		}
 	    }
 	} else {
 	    # Growing upward
-	    if {$line < $ftoc(pickend)} {
-		Ftoc_RangeHighlight [expr $ftoc(pickend)-1] $line
+	    if {$lineno < $ftoc(pickend)} {
+		Ftoc_RangeHighlight [expr $ftoc(pickend)-1] $lineno
 	    } else {
-		if {$line > $ftoc(pickstart)} {
+		if {$lineno > $ftoc(pickstart)} {
 		    if {$ftoc(pickstart) != $ftoc(pickend)} {
 			# Change direction
 			FtocRangeClear [expr $ftoc(pickstart)-1] \
 					$ftoc(pickend)
 		    }
-		    Ftoc_RangeHighlight [expr $ftoc(pickstart)+1] $line
+		    Ftoc_RangeHighlight [expr $ftoc(pickstart)+1] $lineno
 		} else {
 		    # Shrink selection
-		    FtocRangeClear [expr $line-1] $ftoc(pickend)
+		    FtocRangeClear [expr $lineno-1] $ftoc(pickend)
 		}
 	    }
 	}
     } else {
 	if {$ftoc(pickstart) < $ftoc(pickend)} {
 	    # Growing downward
-	    if {$line > $ftoc(pickend)} {
-		FtocRangeInvert [expr $ftoc(pickend)+1] $line
+	    if {$lineno > $ftoc(pickend)} {
+		FtocRangeInvert [expr $ftoc(pickend)+1] $lineno
 	    } else {
-		if {$line < $ftoc(pickstart)} {
+		if {$lineno < $ftoc(pickstart)} {
 		    if {$ftoc(pickstart) != $ftoc(pickend)} {
 			# Change direction
 			FtocRangeInvert [expr $ftoc(pickstart)+1] \
 					$ftoc(pickend)
 		    }
-		    FtocRangeInvert [expr $ftoc(pickstart)-1] $line
+		    FtocRangeInvert [expr $ftoc(pickstart)-1] $lineno
 		} else {
 		    # Shrink selection
-		    FtocRangeInvert [expr $line+1] $ftoc(pickend)
+		    FtocRangeInvert [expr $lineno+1] $ftoc(pickend)
 		}
 	    }
 	} else {
 	    # Growing upward
-	    if {$line < $ftoc(pickend)} {
-		FtocRangeInvert [expr $ftoc(pickend)-1] $line
+	    if {$lineno < $ftoc(pickend)} {
+		FtocRangeInvert [expr $ftoc(pickend)-1] $lineno
 	    } else {
-		if {$line > $ftoc(pickstart)} {
+		if {$lineno > $ftoc(pickstart)} {
 		    if {$ftoc(pickstart) != $ftoc(pickend)} {
 			# Change direction
 			FtocRangeInvert [expr $ftoc(pickstart)-1] \
 					$ftoc(pickend)
 		    }
-		    FtocRangeInvert [expr $ftoc(pickstart)+1] $line
+		    FtocRangeInvert [expr $ftoc(pickstart)+1] $lineno
 		} else {
 		    # Shrink selection
-		    FtocRangeInvert [expr $line-1] $ftoc(pickend)
+		    FtocRangeInvert [expr $lineno-1] $ftoc(pickend)
 		}
 	    }
 	}
     }
-    set ftoc(pickend) $line
+    set ftoc(pickend) $lineno
 }
 proc FtocRangeInvert { start end } {
     global exwin
@@ -337,12 +332,12 @@ proc FtocRangeInvert { start end } {
     if {$start > $end} {
 	set tmp $start ; set start $end ; set end $tmp
     }
-    for {set line $start} {$line <= $end} {incr line} {
+    for {set lineno $start} {$lineno <= $end} {incr lineno} {
 	catch {
 	    set newtag range
 	    set oldtag {}
 	    set nuke 0
-	    foreach tag [$win tag names $line.0] {
+	    foreach tag [$win tag names $lineno.0] {
 		case $tag {
 		    deleted { set newtag drange ; set oldtag $tag ; break; }
 		    moved { set newtag mrange ; set oldtag $tag ; break; }
@@ -354,16 +349,16 @@ proc FtocRangeInvert { start end } {
 		}
 	    }
 	    if {$nuke} {
-		set ix [lsearch $ftoc(lineset) $line]
+		set ix [lsearch $ftoc(lineset) $lineno]
 		if {$ix >= 0} {
 		    set ftoc(lineset) [lreplace $ftoc(lineset) $ix $ix]
 		}
 	    }
 	    if {$oldtag != {}} {
-		$win tag remove $oldtag $line.0 $line.end
+		$win tag remove $oldtag $lineno.0 $lineno.end
 	    }
 	    if {$newtag != {}} {
-		$win tag add $newtag $line.0 $line.end
+		$win tag add $newtag $lineno.0 $lineno.end
 	    }
 	}
     }
@@ -374,34 +369,34 @@ proc Ftoc_RangeHighlight { start end } {
     if {$start > $end} {
 	set tmp $start ; set start $end ; set end $tmp
     }
-    for {set line $start} {$line <= $end} {incr line} {
+    for {set lineno $start} {$lineno <= $end} {incr lineno} {
 	set newtag range
-	foreach tag [$win tag names $line.0] {
+	foreach tag [$win tag names $lineno.0] {
 	    case $tag {
 		{drange deleted} { set newtag drange ;  break; }
 		{mrange moved} { set newtag mrange ;  break; }
 	    }
 	}
-	$win tag add $newtag $line.0 $line.end
+	$win tag add $newtag $lineno.0 $lineno.end
     }
 }
-proc Ftoc_LinesHighlight { lines } {
+proc Ftoc_LinesHighlight { linenos } {
     global exwin
     set win $exwin(ftext)
-    if {$lines == {}} {
+    if {$linenos == {}} {
 	return
     }
-    WidgetTextYview $exwin(ftext) -pickplace [lindex $lines 0].0
+    WidgetTextYview $exwin(ftext) -pickplace [lindex $linenos 0].0
     update idletasks
-    foreach line $lines {
+    foreach lineno $linenos {
 	set newtag range
-	foreach tag [$win tag names $line.0] {
+	foreach tag [$win tag names $lineno.0] {
 	    case $tag {
 		{drange deleted} { set newtag drange ;  break; }
 		{mrange moved} { set newtag mrange ;  break; }
 	    }
 	}
-	$win tag add $newtag $line.0 $line.end
+	$win tag add $newtag $lineno.0 $lineno.end
     }
 }
 proc FtocRangeClear { start end } {
@@ -410,26 +405,26 @@ proc FtocRangeClear { start end } {
     if {$start > $end} {
 	set tmp $start ; set start $end ; set end $tmp
     }
-    for {set line $start} {$line <= $end} {incr line} {
+    for {set lineno $start} {$lineno <= $end} {incr lineno} {
 	catch {
 	    set newtag {}
 	    set oldtag range
-	    foreach tag [$win tag names $line.0] {
+	    foreach tag [$win tag names $lineno.0] {
 		case $tag {
 		    drange { set newtag deleted ; set oldtag drange; break; }
 		    mrange { set newtag moved ; set oldtag mrange; break; }
 		    range { break }
 		}
 	    }
-	    $win tag remove $oldtag $line.0 $line.end
+	    $win tag remove $oldtag $lineno.0 $lineno.end
 	    if {$newtag != {}} {
-		$win tag add $newtag $line.0 $line.end
+		$win tag add $newtag $lineno.0 $lineno.end
 	    }
 	}
     }
 }
 proc Ftoc_RangeUnHighlight { } {
-    global exwin
+    global exwin exmh
     set win $exwin(ftext)
     foreach tag {range drange mrange} {
 	foreach range [FtocMakePairs [$win tag ranges $tag]] {
@@ -454,77 +449,116 @@ proc Ftoc_BindRight { cmd } {
     bind $exwin(ftext) <3> $cmd
 }
 
-proc Ftoc_FindMsg { msgid {line {}} } {
-    global ftoc
-    if {$line != {}} {
-	switch -glob -- $line {
-	    first  {return 1}
-	    last   {return $ftoc(numMsgs)}
-	    [0-9]* {
-		if {$line > $ftoc(numMsgs)} {
-		    return $ftoc(numMsgs)
-		} else {
-		    return $line
-		}
-	    }
-	    default {return {}}
+proc Ftoc_FindMsgs {msgids} {
+    global ftoc msgtolinecache
+    set linenos {}
+    foreach msg $msgids {
+	set lineno [Ftoc_FindMsg $msg]
+	if {$lineno != {}} {
+	    lappend linenos $lineno
 	}
     }
+    return $linenos
+}
+proc Ftoc_FindMsg { msgid } {
+    global ftoc msgtolinecache
     if {$msgid == {}} {
-	return {}
+        return {}
+    }
+    if {![catch {set msgtolinecache($msgid)} lineno]} {
+        return $lineno
     }
-#
-# Linear search for pick and thread FTOCs (pseudo-displays)
-#
     if !$ftoc(displayValid) {
-        for {set L 1} {$L <= $ftoc(numMsgs)} {incr L} {
-            if {[Ftoc_MsgNumber $L] == $msgid} {
-                return $L
+        #
+        # Linear search for pick and thread FTOCs (pseudo-displays)
+        #
+        for {set lineno 1} {$lineno <= $ftoc(numMsgs)} {incr lineno} {
+            if {[Ftoc_MsgNumber $lineno] == $msgid} {
+                return $lineno
             }
         }
         return {}
     }
 
-    set min 1
-    set max $ftoc(numMsgs)	;# Ignore trailing blank line
+    #
+    # Binary search for other FTOCs
+    #
+    set minlineno 1
+    set minmsgid [Ftoc_MsgNumber $minlineno]
+    if {$msgid == $minmsgid} {
+        return $minlineno
+    }
+    set maxlineno $ftoc(numMsgs)  ;# Ignore trailing blank line
+    set maxmsgid [Ftoc_MsgNumber $maxlineno]
+    if {$msgid == $maxmsgid} {
+        return $maxlineno
+    }
     while (1) {
-	set m1 [Ftoc_MsgNumber $min]
-	if {$msgid == $m1} {
-	    return $min
-	}
-	set m2 [Ftoc_MsgNumber $max]
-	if {$msgid == $m2} {
-	    return $max
-	}
-	if {$msgid > $m2 || $msgid < $m1} {
-	    Exmh_Status "Cannot find $msgid ($m1,$m2)" warn
-	    return "" ;# new message not listed
-	}
-	if {$max == $min} {
-	    return ""	;# not found
-	}
-	set next [expr int(($max+$min)/2)]
-	set m3 [Ftoc_MsgNumber $next]
-	if {$m3 > $msgid} {
-	    set max $next
-	} elseif {$min == $next} {
-	    Exmh_Status "Cannot find $msgid" warn
-	    return "" ;# new message not listed
-	} else {
-	    set min $next
-	}
+        if {$msgid > $maxmsgid || $msgid < $minmsgid} {
+            Exmh_Status "Cannot find $msgid ($minmsgid,$maxmsgid)" warn
+            set msgtolinecache($msgid) {}
+            return {} ;# new message not listed
+        }
+        if {$maxlineno == $minlineno} {
+            set msgtolinecache($msgid) {}
+            return {}   ;# not found
+        }
+        #set nextlineno [expr int(($maxlineno+$minlineno)/2)]
+        # Don't divide in two, guestimate where the line might be instead
+        set nextlineno [expr int($minlineno+1+($msgid-$minmsgid)*($maxlineno-$minlineno-2)/($maxmsgid-$minmsgid))]
+        set nextmsgid [Ftoc_MsgNumber $nextlineno]
+        # Note that a side effect of Ftoc_MsgNumber was to put this entry in 
+	# the cache,so we don't have to do it here.
+        if {$nextmsgid == $msgid} {
+            return $nextlineno
+        } elseif {$nextmsgid > $msgid} {
+            set maxlineno $nextlineno
+            set maxmsgid $nextmsgid
+        } elseif {$minlineno == $nextlineno} {
+            Exmh_Status "Cannot find $msgid" warn
+            set msgtolinecache($msgid) {}
+            return {} ;# new message not listed
+        } else {
+            set minlineno $nextlineno
+            set minmsgid $nextmsgid
+        }
     }
     # not reached
 }
-proc Ftoc_MsgNumber { L } {
-    global exwin
-    if [catch {$exwin(ftext) get $L.0 $L.end} line] {
-	return ""
+proc Ftoc_ClearMsgCache {} {
+    global linetomsgcache msgtolinecache
+    foreach x {linetomsgcache msgtolinecache} {
+	if {[info exists $x]} {
+	    unset $x
+	}
     }
-    return [Ftoc_MsgNumberRaw $line]
+}
+proc Ftoc_MsgNumbers { linenos } {
+    global exwin linetomsgcache msgtolinecache
+    set msgids {}
+    foreach lineno $linenos {
+	set msgid [Ftoc_MsgNumber $lineno]
+	if {$msgid != {}} {
+	    lappend msgids $msgid
+	}
+    }
+    return $msgids
+}
+proc Ftoc_MsgNumber { lineno } {
+    global exwin linetomsgcache msgtolinecache
+    if [catch {set linetomsgcache($lineno)} msgid] {
+        if [catch {$exwin(ftext) get $lineno.0 $lineno.end} line] {
+            set linetomsgcache($lineno) {}
+            return {}
+        }
+        set msgid [Ftoc_MsgNumberRaw $line]
+        set msgtolinecache($msgid) $lineno
+        set linetomsgcache($lineno) $msgid
+    }
+    return $msgid
 }
 proc Ftoc_MsgNumberRaw { line } {
-    if [regexp {( *)([0-9]+)} $line foo foo2 number] {
+    if [regexp {^( *)([0-9]+)} $line foo foo2 number] {
 	return $number
     } else {
 	return ""
@@ -545,12 +579,12 @@ proc FtocPickRange { {addcurrent 0} } {
 	set ftoc(curLine) {}
     }
     foreach range [concat \
-	[FtocMakePairs [$exwin(ftext) tag ranges range]] \
-	[FtocMakePairs [$exwin(ftext) tag ranges drange]] \
-	[FtocMakePairs [$exwin(ftext) tag ranges mrange]]] {
+		       [FtocMakePairs [$exwin(ftext) tag ranges range]] \
+		       [FtocMakePairs [$exwin(ftext) tag ranges drange]] \
+		       [FtocMakePairs [$exwin(ftext) tag ranges mrange]]] {
 	set mark1 [lindex $range 0]
-	set line [lindex [split $mark1 .] 0]
-	lappend lineset $line
+	set lineno [lindex [split $mark1 .] 0]
+	lappend lineset $lineno
     }
     if {$lineset == {}} {
 	return			;# spurious <ButtonRelease-1> events
@@ -568,26 +602,25 @@ proc FtocPickRange { {addcurrent 0} } {
 }
 proc Ftoc_PickSize {} {
     global ftoc
-    set len [llength $ftoc(lineset)]
-    if {$len == 0} {
+    if {$ftoc(curLine) != {}} {
 	return [llength $ftoc(curLine)]
     } else {
-	return $len
+	return [llength $ftoc(lineset)]
     }
 }
-proc Ftoc_NewFtoc {} {
+proc Ftoc_NewFtoc {{linenos ftoclineset}} {
     global ftoc
-    set ids {}
-    foreach l $ftoc(lineset) {
-	lappend ids [Ftoc_MsgNumber $l]
+    if {$linenos == "ftoclineset"} {
+	set linenos $ftoc(lineset)
     }
-    if {[llength $ids] <= 1} {
+    set msgids [Ftoc_MsgNumbers $linenos]
+    if {[llength $msgids] <= 1} {
 	Exmh_Status "Select more than one message first" warn
 	return
     }
     if {[Ftoc_Changes "new ftoc"] == 0} {
-	Exmh_Status $ids
-	Scan_ProjectSelection $ids
+	Exmh_Status $msgids
+	Scan_ProjectSelection $msgids
     }
 }
 
@@ -599,26 +632,27 @@ proc Ftoc_ClearCurrent {} {
     global ftoc exwin
     set ftoc(pickone) 1
     set ftoc(lineset) {}
-
+    
+    if {$ftoc(curLine) == {}} {
+	set ftoc(curLine) [Mh_Cur $ftoc(folder)]
+    }
     if {$ftoc(curLine) != {}} {
-	$exwin(ftext) tag remove current $ftoc(curLine).0 $ftoc(curLine).end
-	$exwin(ftext) tag remove currentBg $ftoc(curLine).0 $ftoc(curLine).end
+	$exwin(ftext) tag remove cur $ftoc(curLine).0 $ftoc(curLine).end
 	Ftoc_RescanLine $ftoc(curLine)
     }
     return $ftoc(curLine)
 }
-proc Ftoc_Change { msgid line {show show} } {
-    global ftoc exwin
-    set ftoc(curLine) [Ftoc_FindMsg $msgid $line]
+proc Ftoc_Change { lineno {show show} } {
+    global ftoc exwin mhProfile
+    set ftoc(curLine) $lineno
     if {$ftoc(curLine) == {}} {
 	set ok 0
     } else {
 	if {$show == "show"} {
-	    $exwin(ftext) tag remove unseen $ftoc(curLine).0 $ftoc(curLine).end
+	    $exwin(ftext) tag remove $mhProfile(unseen-sequence) $ftoc(curLine).0 $ftoc(curLine).end
 	}
 	Ftoc_RescanLine $ftoc(curLine) +
-	$exwin(ftext) tag add current $ftoc(curLine).0 $ftoc(curLine).end
-	$exwin(ftext) tag add currentBg $ftoc(curLine).0 $ftoc(curLine).end
+	$exwin(ftext) tag add cur $ftoc(curLine).0 $ftoc(curLine).end
 	set top [$exwin(ftext) index @0,4]
 	if [catch {expr {$top+1}}] {set top 0}	;# trap 100.-1 format, iconic
 	if {$ftoc(curLine) == $top ||
@@ -631,53 +665,49 @@ proc Ftoc_Change { msgid line {show show} } {
     }
     return $ok
 }
-proc Ftoc_ShowUnseen { folder } {
-    global exwin flist
-    set unseen [Flist_UnseenMsgs $folder]
-    if {[llength $unseen] > 0} {
-	set end [$exwin(ftext) index end]
-	set line [lindex [split $end .] 0]
-	set msgNum 0
-	for {} {$line > 0} {incr line -1} {
-	    set msgNum [Ftoc_MsgNumber $line]
-	    set i [lsearch $unseen $msgNum]
-	    if {$i >= 0} {
-		$exwin(ftext) tag add unseen $line.0 $line.end
-		set unseen [lreplace $unseen $i $i]
-		if {[llength $unseen] == 0} {
-		    return 1
-		}
-	    }
-	}
-	# Repair bogus unseen sequences
-	# msgNum is the smallest message number, but it might not be
-	# the first message in the folder because of short scans
-	# Anything in the unseen sequence above msgNum is probably wrong
-	# and can result from races with the background process
-	foreach id $unseen {
-	    if {$id > $msgNum} {
-		Flist_MsgSeen $id
+proc Ftoc_InitSequences { w } {
+    global exwin
+    set seqs [option get . sequences {}]
+    foreach seq $seqs {
+	eval $w tag configure $seq \
+	    [option get . sequence_$seq {}]
+	$w tag raise $seq
+    }
+}
+proc Ftoc_ShowSequence { seq {msgids {}} } {
+    global exwin exmh
+    set seqids [Seq_Msgs $exmh(folder) $seq]
+    if {$msgids != {}} {
+	foreach msg $msgids {
+	    set lineno [Ftoc_FindMsg $msg]
+	    if {[lsearch -exact $seqids $msg] == -1} {
+		$exwin(ftext) tag remove $seq $lineno.0 $lineno.end
+	    } else {
+		$exwin(ftext) tag add $seq $lineno.0 $lineno.end
 	    }
 	}
     } else {
-	Flist_SeenAll $folder	;# clear highlighting
-	return 0
+	$exwin(ftext) tag remove $seq 1.0 end
+	foreach lineno [Ftoc_FindMsgs $seqids] {
+	    $exwin(ftext) tag add $seq $lineno.0 $lineno.end
+	}
     }
 }
-proc Ftoc_MarkSeen { ids } {
-    global exwin ftoc
 
-    set ids [lsort -integer -decreasing $ids]
-    for {set L $ftoc(numMsgs)} {$L > 0} {incr L -1} {
-	set ix [lsearch $ids [Ftoc_MsgNumber $L]]
-	if {$ix >= 0} {
-	    $exwin(ftext) tag remove unseen $L.0 $L.end
-	    set ids [lreplace $ids $ix $ix]
-	    if {[llength $ids] == 0} {
-		break
+proc Ftoc_ShowSequences { {msgids {}} } {
+    global exwin exmh
+    if {$msgids == {}} {
+	set seqs [option get . sequences {}]
+	set hiddenseqs [option get . hiddensequences {}]
+	foreach seq $seqs {
+	    if {[lsearch -exact $hiddenseqs $seq] == -1} {
+		$exwin(ftext) tag remove $seq 1.0 end
 	    }
 	}
     }
+    foreach seq [Mh_Sequences $exmh(folder)] {
+	Ftoc_ShowSequence $seq $msgids
+    }
 }
 proc Ftoc_RescanLine { ix {plus none} } {
     global exmh exwin ftoc
@@ -704,6 +734,7 @@ proc Ftoc_RescanLine { ix {plus none} } {
 		# Here we mark the display dirty to force an update
 		# of the cache and prevent later rescans.
 		set ftoc(displayDirty) 1
+		Ftoc_ClearMsgCache
 	    }
 	}
 	if {$ok} {
@@ -730,18 +761,18 @@ proc Ftoc_NextImplied { {show show} {implied implied} } {
 }
 proc Ftoc_Next { show {implied no} } {
     # Go to the next message in the scan display
-    global exmh flist ftoc
-
+    global exmh flist ftoc mhProfile
+    
     set ftoc(direction) "next"
     if {$ftoc(curLine) == {}} {
-	if [Msg_ShowUnseen] {
+	if [Msg_Show $mhProfile(unseen-sequence)] {
 	    return
 	}
     }
     set next [FtocSkipMarked $ftoc(curLine) 1]
     if {($ftoc(curLine) == $next) || \
-	($ftoc(curLine) >= $ftoc(numMsgs)) || \
-	($ftoc(curLine) <= 0)} {
+	    ($ftoc(curLine) >= $ftoc(numMsgs)) || \
+	    ($ftoc(curLine) <= 0)} {
 	# End of folder
 	Ftoc_NextFolder $implied
     } else {
@@ -751,7 +782,7 @@ proc Ftoc_Next { show {implied no} } {
 }
 proc Ftoc_Prev { {show show} } {
     global ftoc
-
+    
     Exmh_Debug Ftoc_Prev
     if {$ftoc(curLine) == {}} {
 	if {$ftoc(numMsgs) > 0} {
@@ -767,7 +798,7 @@ proc Ftoc_Prev { {show show} } {
     }
 }
 proc Ftoc_NextFolder { {implied no} } {
-    global ftoc exmh
+    global ftoc exmh mhProfile
     # Try to chain to the next folder with unread messages.
     if {$implied != "no"} {
 	# Implied - chained with some other operation - be lenient
@@ -784,18 +815,18 @@ proc Ftoc_NextFolder { {implied no} } {
 	    return
 	}
     }
-    set f [Flist_NextUnseen]
-    if {[string length $f] != 0} {
+    set folder [Flist_NextUnvisited]
+    if {[string length $folder] != 0} {
 	if {$ftoc(softChange)} {
 	    set ftoc(lastFolder) $exmh(folder)
-	    Folder_Change $f Msg_ShowUnseen
+	    Folder_Change $folder [list Msg_Show $mhProfile(unseen-sequence)]
 	    return
 	} else {
 	    set ftoc(softChange) 1
 	    Ftoc_ClearCurrent
 	    Msg_ClearCurrent
 	    Exmh_Status ""
-	    Exmh_Status "End of folder; <Next> => $f" warn
+	    Exmh_Status "End of folder; <Next> => $folder" warn
 	    return
 	}
     }
@@ -817,17 +848,17 @@ proc Ftoc_PrevMarked { {show show} } {
     Ftoc_Prev $show
     set ftoc(skipMarked) $skip
 }
-proc Ftoc_Marked { id } {
+proc Ftoc_Marked { msgid } {
     global ftoc exwin
     if {$ftoc(skipMarked) == 0} {
 	return 0	;# Pretend it isn't marked
     }
-    set i [Ftoc_FindMsg $id]
-    if {[string length $i] == 0} {
+    set lineno [Ftoc_FindMsg $msgid]
+    if {[string length $lineno] == 0} {
 	return 1	;# Can't find it, pretend it's marked
     }
     set marked 0
-    foreach tag [$exwin(ftext) tag names $i.0] {
+    foreach tag [$exwin(ftext) tag names $lineno.0] {
 	if [regexp {(deleted|moved|drange|mrange)} $tag] {
 	    set marked 1 ; break;
 	}
@@ -836,7 +867,7 @@ proc Ftoc_Marked { id } {
 }
 proc FtocSkipMarked {start inc} {
     global exwin ftoc
-
+    
     if {$start == {}} {
 	return {}
     }
@@ -859,7 +890,7 @@ proc FtocSkipMarked {start inc} {
 
 proc Ftoc_Changes {type {allowAuto 1} } {
     global ftoc
-
+    
     if {$ftoc(changed) != 0} then {
 	Exmh_Debug Ftoc_Changes $type
 	if {("$allowAuto" == "1") && $ftoc(autoCommit)} {
@@ -887,8 +918,6 @@ proc Ftoc_Changes {type {allowAuto 1} } {
 	    set ftoc(changed) 0
 	    return 1
 	}
-    } else {
-	Msg_CheckPoint		;# Sync unseen and cur message state.
     }
     return $ftoc(changed)
 }
@@ -920,33 +949,46 @@ proc FtocDialog { changes type } {
     return $ftoc(okToCommit)
 }
 
-proc Ftoc_Iterate { lineVar body } {
+proc Ftoc_CurLines {} {
     global ftoc
-    upvar $lineVar line
-    catch {
-	if {$ftoc(pickone)} {
-	    if {$ftoc(curLine) != {}} {
-		set line $ftoc(curLine)
-		uplevel 1 $body
-	    }
-	} else {
-	    foreach line $ftoc(lineset) {
-		uplevel 1 $body
-	    }
-	}
+    if {$ftoc(curLine) != {}} {
+	return $ftoc(curLine)
+    } elseif {!$ftoc(pickone)} {
+	return $ftoc(lineset);
+    } else {
+	return {}
+    }
+}
+
+proc Ftoc_CurMsgs {} {
+    Ftoc_MsgNumbers [Ftoc_CurLines]
+}
+
+proc Ftoc_Iterate { linenoVar body } {
+    global ftoc
+    upvar $linenoVar lineno
+    foreach lineno [Ftoc_CurLines] {
+	uplevel 1 $body
+    }
+}
+proc Ftoc_MsgIterate { msgidVar body } {
+    global ftoc
+    upvar $msgidVar msgid
+    foreach msgid [Ftoc_CurMsgs] {
+	uplevel 1 $body
     }
 }
 proc Ftoc_Unmark {} {
     global ftoc
-
+    
     set hits 0
-    Ftoc_Iterate line {
-	if [FtocUnmarkInner $line] { incr hits }
+    Ftoc_Iterate lineno {
+	if [FtocUnmarkInner $lineno] { incr hits }
     }
     Exmh_Status "Unmarked $hits msgs"
     incr ftoc(changed) -$hits
 }
-proc FtocUnmarkInner { line {all 0}} {
+proc FtocUnmarkInner { lineno {all 0}} {
     global exwin
     set res 0
     if {$all} {
@@ -954,71 +996,72 @@ proc FtocUnmarkInner { line {all 0}} {
     } else {
 	set pat (deleted|moved|drange|mrange)
     }
-    foreach tag [$exwin(ftext) tag names $line.0] {
+    foreach tag [$exwin(ftext) tag names $lineno.0] {
 	if [regexp $pat $tag] {
-	    $exwin(ftext) tag remove $tag $line.0 $line.end
+	    $exwin(ftext) tag remove $tag $lineno.0 $lineno.end
 	    if [regexp {(drange|mrange|crange)} $tag] {
-		eval $exwin(ftext) tag add range $line.0 $line.end
+		eval $exwin(ftext) tag add range $lineno.0 $lineno.end
 	    }
 	    set res 1
 	}
     }
     return $res
 }
-proc Ftoc_Delete { line msgid } {
+proc Ftoc_Delete { lineno } {
     global exwin ftoc
     $exwin(ftext) configure -state normal
-    $exwin(ftext) delete $line.0 "$line.end + 1 chars"
+    $exwin(ftext) delete $lineno.0 "$lineno.end + 1 chars"
     $exwin(ftext) configure -state disabled
     set ftoc(displayDirty) 1
+    Ftoc_ClearMsgCache
 }
-proc Ftoc_RemoveMark { line msgid } {
+proc Ftoc_RemoveMark { lineno } {
     # Flag the current message(s) for deletion
     global ftoc exwin
-    if ![FtocUnmarkInner $line 1] {
+    if ![FtocUnmarkInner $lineno 1] {
 	incr ftoc(changed)
     }
-
+    
     if {$ftoc(pickone)} {
-	$exwin(ftext) tag add deleted $line.0 $line.end
+	$exwin(ftext) tag add deleted $lineno.0 $lineno.end
     } else {
-	$exwin(ftext) tag remove range $line.0 $line.end
-	$exwin(ftext) tag add drange $line.0 $line.end
+	$exwin(ftext) tag remove range $lineno.0 $lineno.end
+	$exwin(ftext) tag add drange $lineno.0 $lineno.end
     }
 }
-proc Ftoc_MoveMark { line msgid } {
+proc Ftoc_MoveMark { lineno } {
     global ftoc exwin exmh
-    if ![FtocUnmarkInner $line] {
+    if ![FtocUnmarkInner $lineno] {
 	incr ftoc(changed)
     }
     # This tag records the target folder
-    $exwin(ftext) tag add [list moved $exmh(target)] $line.0 $line.end
-
+    $exwin(ftext) tag add [list moved $exmh(target)] $lineno.0 $lineno.end
+    
     if {$ftoc(pickone)} {
-	$exwin(ftext) tag add moved $line.0 $line.end
+	$exwin(ftext) tag add moved $lineno.0 $lineno.end
     } else {
-	$exwin(ftext) tag remove range $line.0 $line.end
-	$exwin(ftext) tag add mrange $line.0 $line.end
+	$exwin(ftext) tag remove range $lineno.0 $lineno.end
+	$exwin(ftext) tag add mrange $lineno.0 $lineno.end
     }
 }
-proc Ftoc_CopyMark { line msgid } {
+proc Ftoc_CopyMark { lineno } {
     global ftoc exwin exmh
-    if ![FtocUnmarkInner $line] {
+    if ![FtocUnmarkInner $lineno] {
 	incr ftoc(changed)
     }
     # This tag records the target folder
-    $exwin(ftext) tag add [list copied $exmh(target)] $line.0 $line.end
-
+    $exwin(ftext) tag add [list copied $exmh(target)] $lineno.0 $lineno.end
+    
     if {$ftoc(pickone)} {
-	$exwin(ftext) tag add moved $line.0 $line.end
+	$exwin(ftext) tag add moved $lineno.0 $lineno.end
     } else {
-	$exwin(ftext) tag remove range $line.0 $line.end
-	$exwin(ftext) tag add mrange $line.0 $line.end
+	$exwin(ftext) tag remove range $lineno.0 $lineno.end
+	$exwin(ftext) tag add mrange $lineno.0 $lineno.end
     }
 }
 proc Ftoc_Commit { rmmCommit moveCommit copyCommit } {
     global ftoc exwin
-
+    
     # Disable operations on ranges
     Ftoc_RangeUnHighlight
     if {! $ftoc(pickone)} {
@@ -1026,7 +1069,7 @@ proc Ftoc_Commit { rmmCommit moveCommit copyCommit } {
 	set ftoc(lineset) {}
 	set ftoc(pickone) 1
     }
-
+    
     Exmh_Status "Committing $ftoc(changed) changes..."
     $exwin(ftext) configure -state normal
     FtocCommit deleted $rmmCommit
@@ -1044,16 +1087,16 @@ proc Ftoc_Commit { rmmCommit moveCommit copyCommit } {
     }
 }
 proc FtocCommit {tagname commitProc {copyCommitProc {}} } {
-    global ftoc exmh exwin msg
-
+    global ftoc exmh exwin msg mhProfile
+    
     set delmsgs {}
     set curid [file tail $msg(path)]
     set pairs [FtocMakeReversePairs [$exwin(ftext) tag ranges $tagname]]
     foreach range $pairs {
 	set c0 [lindex $range 0]
 	set ce [lindex $range 1]
-	scan $c0 "%d" L
-	set msgid [Ftoc_MsgNumber $L]
+	scan $c0 "%d" lineno
+	set msgid [Ftoc_MsgNumber $lineno]
 	set F {}
 	set delline 0	;# Nuke display line
 	foreach tag [$exwin(ftext) tag names $c0] {
@@ -1090,19 +1133,19 @@ proc FtocCommit {tagname commitProc {copyCommitProc {}} } {
 	    lappend delmsgs $msgid
 	    set delline 1
 	}
-	Flist_MsgSeen $msgid	;# in case deleted or moved w/out viewing
+	Seq_Del $exmh(folder) $mhProfile(unseen-sequence) $msgid	;# in case deleted or moved w/out viewing
 	if {$delline} {
-	    Msg_UnSeen $msgid	;# avoid MH mark bug
 	    $exwin(ftext) delete $c0 "$ce + 1 chars"
 	    set ftoc(displayDirty) 1
+	    Ftoc_ClearMsgCache
 	    if {$msgid == $curid} {
 		Ftoc_ClearCurrent
 		Msg_ClearCurrent
 	    }
-	    if {$L == $ftoc(curLine)} {
+	    if {$lineno == $ftoc(curLine)} {
 		set ftoc(curLine) {}
 	    } elseif {$ftoc(curLine) != {}} {
-		if {$L < $ftoc(curLine)} {
+		if {$lineno < $ftoc(curLine)} {
 		    incr ftoc(curLine) -1
 		    if {$ftoc(curLine) == 0} {
 			set ftoc(curLine) {}
@@ -1111,7 +1154,7 @@ proc FtocCommit {tagname commitProc {copyCommitProc {}} } {
 	    }
 	    incr ftoc(numMsgs) -1
 	} else {
-	    FtocUnmarkInner $L
+	    FtocUnmarkInner $lineno
 	}
 	incr ftoc(changed) -1
     }
@@ -1172,15 +1215,11 @@ proc FtocMakeReversePairs { list } {
     }
 }
 
-proc Ftoc_MoveFeedback { msgid line } {
+proc Ftoc_MoveFeedback { msgid } {
     global exwin ftoc
+    set lineno [Ftoc_FindMsg $msgid]
     set msg [Exmh_OldStatus]
-    if {[string compare $line "last"] == 0} {
-	set line $ftoc(numMsgs)
-    } elseif {[string compare $line first] == 0} {
-	set line 1
-    }
-    foreach tag [$exwin(ftext) tag names $line.0] {
+    foreach tag [$exwin(ftext) tag names $lineno.0] {
 	if [regexp {moved (.+)} $tag match folder] {
 	    Exmh_Status "$msgid => +$folder"
 	    return
@@ -1222,7 +1261,7 @@ proc Ftoc_FindAll {string} {
 	Ftoc_PickMsgs $msgids 0
 	return 1
     }
-
+    
 }
 proc Ftoc_FindMatch {L string} {
     global exwin ftoc
@@ -1277,18 +1316,18 @@ proc Ftoc_SelectAll {} {
 proc Ftoc_SelectAllToEnd {} {
     global ftoc
     if {$ftoc(curLine) != {}} {
-      FtocRangeStart $ftoc(curLine)
+	FtocRangeStart $ftoc(curLine)
     } else {
-      if {$ftoc(direction) == "next"} {
-        FtocRangeStart 1
-      } else {
-        FtocRangeStart $ftoc(numMsgs)
-      }
+	if {$ftoc(direction) == "next"} {
+	    FtocRangeStart 1
+	} else {
+	    FtocRangeStart $ftoc(numMsgs)
+	}
     }
     if {$ftoc(direction) == "next"} {
-      FtocRangeEnd $ftoc(numMsgs)
+	FtocRangeEnd $ftoc(numMsgs)
     } else {
-      FtocRangeEnd 1
+	FtocRangeEnd 1
     }
 }
 proc Ftoc_CatchUp {} {
@@ -1315,50 +1354,51 @@ set ftocDrag(format,filename) string
 set ftocDrag(type,string) foldermsg
 
 # Drag Selected
-proc FtocDragSelectOld {w x y wx wy} {
-	global exmh ftoc
-
-        set folder $ftoc(folder)
-        if !$ftoc(displayValid) {
-            set folder $exmh(folder)
-        }
-	set line [lindex [split [$w index current] .] 0]
-	set msg [Ftoc_MsgNumber $line]
-	if {$msg == {} || $msg == 0} return
-
-	# Hand off to Drag code
-	global ftocDrag mhProfile
-	set ftocDrag(source) $w
-	set ftocDrag(data,foldermsg) "+$folder $msg"
-	set ftocDrag(data,filename) $mhProfile(path)/$folder/$msg
-	Drag_Source ftocDrag $x $y
-}
 proc FtocDragSelect {w x y wx wy} {
     global exmh ftoc ftocDrag mhProfile
-
+    
     set folder $ftoc(folder)
     if !$ftoc(displayValid) {
-        set folder $exmh(folder)
+	set folder $exmh(folder)
     }
-    set msgs {}
     if $ftoc(pickone) {
-	set line [lindex [split [$w index current] .] 0]
-	set msgs [Ftoc_MsgNumber $line]
-	if {$msgs == {} || $msgs == 0} return
-	set ftocDrag(data,filename) $mhProfile(path)/$folder/$msgs
+	set lineno [lindex [split [$w index cur] .] 0]
+	set msgids [Ftoc_MsgNumber $lineno]
+	if {$msgids == {} || $msgids == 0} return
+	set ftocDrag(data,filename) $mhProfile(path)/$folder/$msgids
     } else {
-	foreach line $ftoc(lineset) {
-	    set msgid [Ftoc_MsgNumber $line]
-	    if {$msgid != {}} {
-		lappend msgs $msgid
-	    }
-	}
-    	catch {unset ftocDrag(data,filename)}
+	set msgids [Ftoc_MsgNumber $ftoc(lineset)]
+	catch {unset ftocDrag(data,filename)}
     }
-
+    
     # Hand off to Drag code
     set ftocDrag(source) $w
-    set ftocDrag(data,foldermsg) "+$folder $msgs"
+    set ftocDrag(data,foldermsg) "+$folder $msgids"
     Drag_Source ftocDrag $x $y
-  }
-
+}
+proc FtocToggleSequence { seq } {
+    global ftoc exmh
+    set folder $exmh(folder)
+    set origmsgids [Seq_Msgs $folder $seq]
+    set selmsgids [Ftoc_CurMsgs]
+    if {$selmsgids == {}} {
+	Exmh_Status "No messages were selected"
+    } else {
+	# If any selected message is not already in the sequence, then add 
+	#   messages to the sequence.
+	# If all selected messages are already in the sequence, then remove
+	#   messages from the sequence.
+	set flag del
+	foreach msgid $selmsgids {
+	    if {[lsearch -exact $origmsgids $msgid] == -1} {
+		set flag add
+	    }
+	}
+	if {$flag == "del"} {
+	    Seq_Del $folder $seq $selmsgids
+	} else {
+	    Seq_Add $folder $seq $selmsgids
+	}
+	Ftoc_ShowSequence $seq $selmsgids
+    }
+}
