@@ -76,7 +76,7 @@ proc Inc_Startup {} {
        ($inc(startupflist) == "default" &&
        (! $inc(onStartup) || $inc(style) == "multidrop"))} {
 	Exmh_Status "Checking folders"
-	Flist_FindUnseen
+	Flist_FindSeqs
     }
     if {$inc(onStartup)} {
 	set s [Sound_Off]
@@ -97,13 +97,13 @@ proc Inc_Mapped {} {
 # be the first one just inc'ed.)
 
 proc Inc_Show {} {
-    global exmh
+    global exmh mhProfile
 
     Inc
     if { $exmh(folder) != "inbox" } {
 	Folder_Change inbox
     }
-    Msg_ShowUnseen
+    Msg_Show $mhProfile(unseen-sequence)
 }
 
 proc Inc {} {
@@ -190,7 +190,7 @@ proc Inc_Expect {cmd} {
 }
 
 proc Inc_InboxFinish { f incout {hook {}} } {
-    global exmh
+    global exmh mhProfile
     set msgids {}
     Scan_Iterate $incout line {
 	set id [Ftoc_MsgNumberRaw $line]
@@ -203,7 +203,7 @@ proc Inc_InboxFinish { f incout {hook {}} } {
 	Exmh_Status "No new messages in $f"
 	return
     }
-    Flist_AddUnseen $f $msgids
+    Seq_Add $f $mhProfile(unseen-sequence) $msgids
     if {$hook != {}} {
 	eval $hook
     }
@@ -235,16 +235,16 @@ proc Inc_Presort {{doinc 1}} {
 	    set user ""
 	}
     }
-    if {[catch {glob $mhProfile(path)/MyIncTmp/*} files] == 0} {
+    if {[catch {lsort -dictionary [glob $mhProfile(path)/MyIncTmp/*]} files] == 0} {
 	Exmh_Status "incfilter ..."
 	foreach file $files {
 	    if {![regexp {^[0-9]+$} [file tail $file]]} {
-		Exmh_Status "Deleting stray file in MyIncTmp: [file tail $file]" red 
+		Exmh_Status "Deleting stray file in MyIncTmp: [file tail $file]" warning
 		File_Delete $file
 		continue
 	    }
 	    if {$inc(presortFeedback)} {
-		if [catch {exec grep "^Subject:" $file | head -1} \
+		if [catch {exec grep -i "^Subject:" $file | head -1} \
 		    subject] {
 		    set subject ""
 		}
@@ -279,7 +279,7 @@ proc Inc_Presort {{doinc 1}} {
 		Mh_Refile MyIncTmp [file tail $file] MyIncErrors 
 	    } else {
 		#
-		# print the messages we got (Brent, delete if you don't like)
+		# print the messages we got
 		#
 
 		foreach line [split $err "\n"] {
@@ -290,9 +290,9 @@ proc Inc_Presort {{doinc 1}} {
 		if {$inc(presortFeedback)} {
 		    if {[string length $subject] > 0} {
 			regsub "^Subject: *" $subject "" subject
-			Exmh_Status "[file tail $file]: $subject" red
+			Exmh_Status "[file tail $file]: $subject" warning
 		    } else {
-			Exmh_Status [file tail $file] red
+			Exmh_Status [file tail $file] warning
 		    }
 		}
 	    }
@@ -301,7 +301,7 @@ proc Inc_Presort {{doinc 1}} {
     File_Delete $mhProfile(path)/MyIncTmp/$mhProfile(mh-sequences)
     catch {exec rmdir $mhProfile(path)/MyIncTmp}
 
-    Flist_FindUnseen		;# Needed to set new message state.
+    Flist_FindSeqs		;# Needed to set new message state.
     # This after breaks a potential deadlock:
     # UI Inc button is pressed - registers outstanding Inc operation
     # PresortFinish notes new messages in the current folder and wants
@@ -311,10 +311,10 @@ proc Inc_Presort {{doinc 1}} {
     after 1 {BgRPC Inc_PresortFinish}
 }
 proc Inc_PresortFinish {} {
-    global exmh ftoc
+    global exmh ftoc mhProfile
     LOG Inc_PresortFinish
-    Mh_Folder $exmh(folder)	;# prestort inc has changed this to MyIncTmp
-    if {$ftoc(displayValid) && [Flist_NumUnseen $exmh(folder)] > 0} {
+    Mh_Folder $exmh(folder)	;# presort inc has changed this to MyIncTmp
+    if {$ftoc(displayValid) && [Seq_Count $exmh(folder) $mhProfile(unseen-sequence)] > 0} {
 	Label_Folder $exmh(folder)
 	Scan_Folder $exmh(folder) $ftoc(showNew)
     }
@@ -327,13 +327,13 @@ proc Inc_PresortFinish {} {
 proc Inc_All {{updateScan 1}} {
     global exdrops exmh ftoc pop
 
-    Exmh_Status "Checking dropboxes..." red
+    Exmh_Status "Checking dropboxes..." warning
     MhSetMailDrops	;# Update, if needed
 
     set hits {}
     foreach record [array names exdrops] {
         set fields [ scan $exdrops($record) "%s %s %s" folder dropname popname]
-        Exmh_Status "dropbox... $dropname" red
+        Exmh_Status "dropbox... $dropname" warning
 	if {[string first / $dropname] < 0} {
 
 	    # Not a pathname, but a POP hostname
@@ -374,7 +374,7 @@ proc Inc_All {{updateScan 1}} {
     }
     if {$updateScan && ([llength $hits] > 0)} {
 	BgRPC Flist_Done
-	Exmh_Status "New mail in: $hits" blue
+	Exmh_Status "New mail in: $hits"
     } else {
 	Exmh_Status ""
     }
@@ -391,9 +391,9 @@ proc Inc_Pending {} {
     }
     if [llength $active] {
 	# Output the list of active folder
-	Exmh_Status "Active: $active" blue
+	Exmh_Status "Active: $active"
     } else {
-	Exmh_Status "No active folders" red
+	Exmh_Status "No active folders" warning
     }
-    Flist_FindUnseen
+    Flist_FindSeqs
 }
