@@ -14,6 +14,7 @@
 proc Exmh {} {
     global exmh argv 
 
+    Tcl_Tk_Vers_Init	;# Do per-release Tcl/Tk setup here
     Mh_Init		;# Defines mhProfile and identifies mh vs nmh
 
     Preferences_Init "~/.exmh/exmh-defaults" "$exmh(library)/app-defaults"
@@ -80,7 +81,7 @@ is displayed to debug the flist module."}
     Inc_Init
     Exwin_Init
     Flist_Init
-    UnseenWin_Init
+    Seq_Init
     Fcache_Init
     Fdisp_Init		;# After Flist and Fcache
     Sound_Init
@@ -96,7 +97,6 @@ is displayed to debug the flist module."}
     Quote_Init
 
     wm protocol . WM_DELETE_WINDOW Exmh_Done
-    wm protocol . WM_SAVE_YOURSELF [list Exmh_Done 0]
     Exwin_Layout
     if [catch {User_Layout} err] {
 	global errorInfo
@@ -113,6 +113,13 @@ is displayed to debug the flist module."}
     bind . <Map> {ExmhMapped %W}
 
     Folder_Change $exmh(folder)
+
+    # Do this late because the WM seems to call the SAVE_YOURSELF hook
+    # and we want to make sure we are in the current folder before
+    # we checkpoint state.  Used to loose the current message because
+    # this was done too early.
+    wm protocol . WM_SAVE_YOURSELF [list Exmh_Done 0]
+
     busy ExmhJunk
 }
 proc ExmhJunk {} {
@@ -211,7 +218,8 @@ proc ExmhResources {} {
 	Preferences_Resource exmh(c_st_normal) c_st_normal blue
 	Preferences_Resource exmh(c_st_error) c_st_error purple
 	Preferences_Resource exmh(c_st_warn) c_st_warn red
-	Preferences_Resource exmh(c_st_background) c_st_background black
+	Preferences_Resource exmh(c_st_bg_msgs) c_st_bg_msgs "medium sea green"
+	Preferences_Resource exmh(c_st_background) c_st_background "\#d9d9d9"
     } else {
 	Preferences_Resource exmh(c_st_normal) c_st_normal black
 	if {$exmh(c_st_normal) != "white" && $exmh(c_st_normal) != "black"} {
@@ -224,18 +232,14 @@ proc ExmhResources {} {
 }
 
 proc Exmh_Status {string { level normal } } {
-    global exmh exwin
+    global exmh exwin tk_version
     if {[string compare $string 0] == 0 } { set string $exmh(version) }
     if [info exists exwin(status)] {
 	switch -- $level {
-	    warn	-
-	    error	-
-	    background	-
+	    warn	{ # do nothing }
+	    error	{ # do nothing }
+	    background	{set level bg_msgs}
 	    normal	{ # do nothing }
-	    red		{set level warn}
-	    blue	{set level normal}
-	    purple	{set level error}
-	    "medium sea green" {set level background}
 	    default	{set level normal}
 	}
 	if ![info exists exmh(c_st_$level)] {
@@ -245,7 +249,16 @@ proc Exmh_Status {string { level normal } } {
 	catch {$exwin(status) configure -fg $exmh(c_st_$level)}
 	$exwin(status) delete 0 end
 	$exwin(status) insert 0 $string
-	$exwin(status) configure -state disabled
+	# Oh, the inhumanity.. backward-incompatible behavior changes
+	if [info exists tk_version] {
+	    if {$tk_version > "8.3"} {
+		# get the readonlyBackground to match the regular one...
+		set state_color [lindex [ $exwin(status) configure -background ] 4 ]
+		$exwin(status) configure -state readonly -readonlybackground $state_color
+	    } else {
+		$exwin(status) configure -state disabled
+	    }
+	}
 	ExmhLog $string
 	update idletasks
     } else {
@@ -264,7 +277,6 @@ proc Exmh_OldStatus {} {
 proc Exmh_CheckPoint {} {
     # This is really "folder change" CheckPoint
     Exmh_Debug Scan_CacheUpdate [time Scan_CacheUpdate]
-    Msg_CheckPoint
 }
 
 proc Exmh_Done {{exit 1}} {
@@ -276,13 +288,13 @@ proc Exmh_Done {{exit 1}} {
 	    catch {exec date} d
 	    Audit "Quit $d"
 	}
-	Exmh_Status "Checkpointing state" red
+	Exmh_Status "Checkpointing state" warning
 	if [info exists exmh(newuser)] {
 	    PreferencesSave nodismiss	;# Save tuned parameters
 	    unset exmh(newuser)
 	}
 	# The following is done in response to WM_SAVE_YOURSELF
-	foreach cmd {Msg_CheckPoint Sedit_CheckPoint Aliases_CheckPoint
+	foreach cmd {Sedit_CheckPoint Aliases_CheckPoint
 		    Exmh_CheckPoint Fcache_CheckPoint	    
 		    Exwin_CheckPoint } {
 	    if {[info command $cmd] != {}} {
@@ -376,6 +388,22 @@ proc ExmhLog { stuff } {
 	catch {
 #	    $exmh(log) insert end " [bw_delta] "
 	    $exmh(log) insert end [clock format [clock seconds] -format "%H:%M:%S "]
+            global tcl_version
+            if {$tcl_version >= 8.3} {
+                set sec [clock seconds]
+                set now [clock clicks -milliseconds]
+                if {[info exist exmh(logLastClicks)]} {
+                    set delta [expr {$now - $exmh(logLastClicks)}]
+                    set delta_sec [expr {$sec - $exmh(logLastSeconds)}]
+                    if {$delta < 0} {
+                        incr delta_sec -1
+                        set delta [expr 1000000 - $delta]
+                    }
+                    $exmh(log) insert end "([format %d.%.06d $delta_sec $delta]) "
+                }
+                set exmh(logLastClicks) $now
+                set exmh(logLastSeconds) $sec
+            }
 	    $exmh(log) insert end $stuff
 	    $exmh(log) insert end \n
 	    if {$exmh(logYview)} {
@@ -440,9 +468,9 @@ proc ExmhLogSave {} {
     if [catch {
 	puts $logfile [$exmh(log) get 1.0 end]
 	close $logfile
-	Exmh_Status "Saved log in [Env_Tmp]/exmhlog.$id" blue
+	Exmh_Status "Saved log in [Env_Tmp]/exmhlog.$id"
     } msg] {
-	Exmh_Status "Cannot save log: $msg" purple
+	Exmh_Status "Cannot save log: $msg" error
     }
 }
 #### Misc
@@ -466,3 +494,20 @@ proc Exmh_DoCommand {} {
     }
     $t see end
 }
+
+proc Tcl_Tk_Vers_Init {} {
+    # Here we do any special tuning needed for specific Tcl/Tk releases
+    # For instance, 8.4a2 and later moved some private variables into
+    # namespaces, so we need to do backward-compatibility until we
+    # fix the code everyplace.
+    global tk_version tk_patchLevel tcl_version tcl_patchLevel
+    if {[info exists tk_version] && ($tk_version > "8.3")} {
+        ::tk::unsupported::ExposePrivateCommand tkEntryBackspace
+        ::tk::unsupported::ExposePrivateCommand tkEntrySeeInsert
+        ::tk::unsupported::ExposePrivateCommand tkMenuUnpost
+        ::tk::unsupported::ExposePrivateCommand tkTextButton1
+        ::tk::unsupported::ExposePrivateCommand tkEntryButton1
+        ::tk::unsupported::ExposePrivateCommand tkTextResetAnchor
+        ::tk::unsupported::ExposePrivateVariable tkPriv
+    }
+}
