@@ -143,12 +143,7 @@ proc Mh_CompSetup {} {
 	if {$indrafts} {
 	    # In drafts with no previously current message
 	    Scan_Folder $exmh(folder)
-	    Msg_Change [Mh_Sequence $exmh(folder) cur]
-	    if {[Mh_Cur $exmh(folder)] == {}} {
-		# Scan_Folder destroyed the cur sequence (drafts must
-		# have been empty). Restore it.
-		Msg_CheckPoint
-	    }
+	    Msg_Change [Seq_Msgs $exmh(folder) cur]
 	}
     }
     set exmh([Mh_Cur $mhProfile(draft-folder)],action) comp
@@ -392,216 +387,287 @@ proc Mh_CurSafe { folder } {
     MhExec folder -pop < /dev/null
     return $cur
 }
-proc Mh_Unseen { folder } {
-    global mhProfile
-    return [Mh_Sequence $folder [lindex [split $mhProfile(unseen-sequence)] 0]]
-}
-proc Mh_MarkSeen { folder ids } {
-    global mhProfile
-    if {[catch {
-	foreach seq [split $mhProfile(unseen-sequence)] {
-	    Mh_SequenceUpdate $folder del $seq $ids
-#	    eval {MhExec mark +$folder -seq $seq -delete} $ids
-	}
-    } err]} {
-	Exmh_Debug Mh_MarkSeen $err
-    }
-}
-proc Mh_MarkUnseen { folder ids } {
-    global mhProfile
-    if {[catch {
-	foreach seq [split $mhProfile(unseen-sequence)] {
-	    Mh_SequenceUpdate $folder add $seq $ids
-#	    eval {MhExec mark +$folder -seq $seq} $ids
-	}
-    } err]} {
-	Exmh_Debug Mh_MarkUnseen $err
-    }
-}
 
-proc Mh_SetCur { f msgid } {
+proc Mh_SetCur { folder msgid } {
     global mhPriv
-    if {[info exists mhPriv(cur,$f)] &&
-	($mhPriv(cur,$f) == $msgid)} {
+    if {[info exists mhPriv(cur,$folder)] &&
+	($mhPriv(cur,$folder) == $msgid)} {
 	return
     }
-    Exmh_Debug Mh_SetCur +$f cur $msgid
-    Mh_SequenceUpdate $f replace cur $msgid
-#    catch {MhExec mark +$f $msgid -seq cur -zero}
-    set mhPriv(cur,$f) $msgid
+    Seq_Set $folder cur $msgid
+    set mhPriv(cur,$folder) $msgid
 }
-proc Mh_Cur { f } {
+proc Mh_Cur { folder } {
     global mhPriv
-    if {[catch {MhCur $f} cur]} {
-	set cur [Mh_CurSafe $f]
+    if {[catch {MhCur $folder} cur]} {
+	set cur [Mh_CurSafe $folder]
     }
-    set mhPriv(cur,$f) $cur
-    return $mhPriv(cur,$f)
+    set mhPriv(cur,$folder) $cur
+    return $mhPriv(cur,$folder)
 }
-proc MhCur { f } {
+proc MhCur { folder } {
     # pick +folder cur changes the context, so we access the files directly
-    global mhProfile mhPriv
-    if {$f == {}} {
+    global mhProfile
+    if {$folder == {}} {
 	return {}
     }
-    set cur [Mh_Sequence $f cur]
-    if {[file exists $mhProfile(path)/$f/$cur]} {
+    set cur [Seq_Msgs $folder cur]
+    if {[file exists $mhProfile(path)/$folder/$cur]} {
 	return $cur
     } else {
 	return {}
     }
 }
-proc Mh_Sequence { f seq } {
-    # pick +folder cur changes the context, so we access the files directly
+proc MhReadSeqs {folder seqsvar} {
     global mhProfile mhPriv
-    set result {}
-    if {[catch {open $mhProfile(path)/$f/$mhProfile(mh-sequences) r} in] == 0} {
-	set old [read $in]
-	close $in
-	foreach line [split $old \n] {
-	    if {[regexp "^$seq: (.*)" $line x msgs]} {
-		return [MhSeqExpand $msgs]
+    upvar $seqsvar seqs
+    # First read the private sequence
+    set mhPriv(changed,private) 0
+    set filename $mhProfile(context)
+    if [file exists $filename] {
+	set mtime [file mtime $filename]
+	if {![info exists mhPriv(privmtime)] || ($mtime != $mhPriv(privmtime))} {
+	    foreach elem [array names mhPriv] {
+		set indices [split $elem ,]
+		if {[lindex $indices 0] == {privseq}} {
+		    if {[lindex $indices 1] == $folder} {
+			unset mhPriv($elem)
+		    }
+		}
+	    }
+	    if {[catch {open $filename r} in] == 0} {
+		Exmh_Debug Reading $filename
+		set old [read $in]
+		close $in
+		foreach line [split $old \n] {
+		    if {$line != {}} {
+			if {[regexp {^([^:]*):\s*(.*)$} $line foo tag msgids]} {
+			    if {[regexp "atr-(.*)-$mhProfile(path)/(.*)" $tag foo seq thisfolder]} {
+				set mhPriv(privseq,$thisfolder,$seq) [MhSeqExpand $thisfolder $msgids]
+				set mhPriv(mode,$seq) private
+			    } else {
+				lappend mhPriv(otherpriv) "$line"
+			    }
+			} else {
+			    Exmh_Status "Bad line in $filename: $line"
+			}
+		    }
+		}
+		set mhPriv(privmtime) $mtime
 	    }
 	}
     }
-    # private sequences
-    if {[catch {open $mhProfile(context) r} in] == 0} {
-	set old [read $in]
-	close $in
-	foreach line [split $old \n] {
-	    set pattern atr-$seq-$mhProfile(path)/$f
-	    # Turn off all special characters in folder name (e.g., c++)
-	    # Thanks to John Farrell
-	    regsub -all {]|[.^$*+|()\[\\]} $pattern {\\&} pattern
-	    if {[regexp "$pattern: (.*)" $line x msgs]} {
-		return [MhSeqExpand $msgs]
+    foreach elem [array names mhPriv] {
+	set indices [split $elem ,]
+	if {[lindex $indices 0] == {privseq}} {
+	    if {[lindex $indices 1] == $folder} {
+		set seqs([lindex $indices 2]) $mhPriv($elem)
+	    }
+	}
+    }
+    # Then read the public sequence
+    set mhPriv(changed,public) 0
+    set filename "$mhProfile(path)/$folder/$mhProfile(mh-sequences)"
+    if [file exists $filename] {
+	set mtime [file mtime $filename]
+	if {![info exists mhPriv(seqmtime,$folder)] || ($mtime != $mhPriv(seqmtime,$folder))} {
+	    foreach elem [array names mhPriv] {
+		set indices [split $elem ,]
+		if {[lindex $indices 0] == {pubseq}} {
+		    if {[lindex $indices 1] == $folder} {
+			unset mhPriv($elem)
+		    }
+		}
+	    }
+	    if {[catch {open $filename r} in] == 0} {
+		Exmh_Debug Reading $filename
+		set old [read $in]
+		close $in
+		foreach line [split $old \n] {
+		    if {$line != {}} {
+			if {[regexp {^([^:]*):\s*(.*)$} $line foo seq msgids]} {
+			    if {[info exists mhPriv(mode,$seq)] && $mhPriv(mode,$seq) == "private" && [info exists mhPriv(pubseq,$folder,$seq)]} {
+				# If this was also in the private file, merge the two
+				# and move to the public file.
+				set mhPriv(changed,private) 1
+				lappend mhPriv(pubseq,$folder,$seq) [MhSeq $folder $seq add $mhPriv(pubseq,$folder,$seq) [MhSeqExpand $folder $msgids]]
+			    } else {
+				set mhPriv(pubseq,$folder,$seq) [MhSeqExpand $folder $msgids]
+			    }
+			    set mhPriv(mode,$seq) public
+			} else {
+			    Exmh_Status "Bad line in $filename: $line"
+			}
+		    }
+		}
+		set mhPriv(seqmtime,$folder) $mtime
+	    }
+	}
+    }
+    foreach elem [array names mhPriv] {
+	set indices [split $elem ,]
+	if {[lindex $indices 0] == {pubseq}} {
+	    if {[lindex $indices 1] == $folder} {
+		set seqs([lindex $indices 2]) $mhPriv($elem)
 	    }
 	}
     }
-    return {}
 }
-proc MhSeqExpand { msgs } {
-    set result {}
-    foreach range [split [string trim $msgs]] {
+
+proc MhGetSeqCache {folder seq } {
+    global mhPriv
+    Exmh_Debug "MhGetSeqCache $folder $seq"
+    set seqlist ""
+    if {[info exists mhPriv(pubseq,$folder,$seq)]} {
+        set seqlist $mhPriv(pubseq,$folder,$seq)
+    }
+    if {[info exists mhPriv(privseq,$folder,$seq)]} {
+        lappend seqlist $mhPriv(privseq,$folder,$seq)
+    }
+    return $seqlist
+}
+
+proc Mh_Sequences { folder } {
+    MhReadSeqs $folder seqs
+    return [array names seqs]
+}
+proc Mh_Sequence { folder seq } {
+    # pick +folder cur changes the context, so we access the files directly
+    MhReadSeqs $folder seqs
+    if [info exists seqs($seq)] {
+	return [MhSeqExpand $folder $seqs($seq)]
+    } else {
+	return {}
+    }
+}
+proc MhSeqExpand { folder sequence } {
+    global mhProfile
+    # Explode a sequence into a list of message numbers
+    set seq {}
+    set rseq {}
+    foreach range [split [string trim $sequence]] {
 	set parts [split [string trim $range] -]
 	if {[llength $parts] == 1} {
-	    lappend result $parts
+	    lappend seq $parts
+	    set rseq [concat $parts $rseq]
 	} else {
 	    for {set m [lindex $parts 0]} {$m <= [lindex $parts 1]} {incr m} {
-		lappend result $m
+		lappend seq $m
+		set rseq [concat $m $rseq]
 	    }
 	}
     }
-    return $result
-}	
-proc Mh_ClearCur { f } {
-    Mh_SequenceUpdate $f clear cur
+    # Hack to weed out sequence numbers for messages that don't exist
+    foreach m $rseq {
+	if ![file exists $mhProfile(path)/$folder/$m] {
+	    Exmh_Debug $mhProfile(path)/$folder/$m not found
+	    set ix [lsearch $seq $m]
+	    set seq [lreplace $seq $ix $ix]
+	} else {
+	    # Real hack
+	    break
+	}
+    }
+    return $seq
 }
 
 # Directly modify the context files to add/remove/clear messages
 # from a sequence
-proc Mh_SequenceUpdate { f how seq {msgs {}} } {
-    global mhProfile
-    Exmh_Debug Mh_SequenceUpdate $f $how $seq $msgs
-    if {[catch {open $mhProfile(path)/$f/$mhProfile(mh-sequences) r} in] == 0} {
-	set old [read $in]
-	set new {}
-	close $in
-	set hit 0
-	foreach line [split $old \n] {
-	    if {[regexp ^$seq: $line]} {
-		switch -- $how {
-		    clear { # Do nothing }
-		    add {
-			append new [MhSeq add $line $msgs]\n
-		    }
-		    del {
-			set tmp [MhSeq del $line $msgs]
-			if {[regexp {[0-9]$ *} $tmp]} {
-			    append new $tmp\n
-			}
-		    }
-		    replace {
-			append new [MhSeq replace $line $msgs]\n
-		    }
-		}
-		set hit 1
-	    } elseif {[string length $line]} {
-		append new $line\n
-	    }
-	}
-	if {! $hit} {
-	    # sequence not found
-	    if {[regexp {(add|replace)} $how]} {
-		append new "$seq: $msgs\n"
-	    } else {
-		return
-	    }
-	}
-	if {[catch {open $mhProfile(path)/$f/$mhProfile(mh-sequences).new w} out] == 0} {
-	    Exmh_Debug New sequences: $mhProfile(path)/$f/$mhProfile(mh-sequences)
-	    Exmh_Debug $new
-	    puts -nonewline $out $new
-	    if {[catch {
-		close $out
-	    } err]} {
-		Exmh_Status "Cannot close $mhProfile(path)/$f/$mhProfile(mh-sequences).new: $err"
-	    } else {
-		Mh_Rename $mhProfile(path)/$f/$mhProfile(mh-sequences).new \
-			$mhProfile(path)/$f/$mhProfile(mh-sequences)
-	    }
-	    return
+proc Mh_SequenceUpdate { folder how seq {msgids {}} {which public}} {
+    global mhProfile mhPriv
+    if {0} {
+	Exmh_Debug Mh_SequenceUpdate $folder $how $seq $msgids $which
+	set l [info level]
+	while {[incr l -1] > 0} {
+	    Exmh_Debug "    : [info level $l]"
 	}
     }
-    # private sequences
-    if {[catch {open $mhProfile(context) r} in] == 0} {
-	if {[catch {open $mhProfile(context).new w} out] == 0} {
-	    set hit 0
-	    while {[gets $in line] >= 0} {
-		if {[string match atr-$seq-$mhProfile(path)/$f:* $line]} {
-		    switch -- $how {
-			clear { # Do nothing }
-			add {
-			    puts $out [MhSeq add $line $msgs] 
-			}
-			del {
-			    puts $out [MhSeq del $line $msgs]
-			}
-			replace {
-			    puts $out [MhSeq replace $line $msgs]
-			}
-		    }
-		    set hit 1
-		} else {
-		    if {$line != {}} {
-			puts $out $line
-		    }
-		}
+    array unset seqs
+    foreach elem [array names mhPriv] {
+	set indices [split $elem ,]
+	if {[lindex $indices 0] == {mode}} {
+	    if {[lindex $indices 1] == $folder} {
+		unset mhPriv($elem)
 	    }
-	    if {! $hit} {
-		if {[regexp {(add|replace)} $how]} {
-		    puts $out "atr-$seq-$mhProfile(path)/$f: $msgs"
-		}
-	    }
-	    close $in
-	    close $out
-	    Mh_Rename $mhProfile(context).new $mhProfile(context)
-	    return
 	}
-	close $in
+    }
+    MhReadSeqs $folder seqs
+    # Set the value for the sequence we're updating
+    if [catch {set seqs($seq)} oldmsgids] {
+	set oldmsgids {}
+    }
+    set seqs($seq) [MhSeq $folder $seq $how $oldmsgids $msgids]
+    if {![catch {set mhPriv(mode,$seq)}] && ($mhPriv(mode,$seq) != $which)} {
+	set mhPriv(changed,$mhPriv(mode,$seq)) 1
+    }
+    set mhPriv(mode,$seq) $which
+    set oldseq [MhSeqMake $oldmsgids]
+    if {$seqs($seq) != $oldseq} {
+	Exmh_Debug "$seq: $oldseq => $seqs($seq)"
+	set mhPriv(changed,$which) 1
+    }
+    if {$mhPriv(changed,public) == 1} {
+	set mhPriv(pubseq,$folder,$seq) [MhSeqExpand $folder $seqs($seq)]
+	set filename $mhProfile(path)/$folder/$mhProfile(mh-sequences)
+	if {[catch {open $filename.new w} out] == 0} {
+	    Exmh_Debug Writing $filename
+	    foreach thisseq [array names seqs] {
+		if {![info exists mhPriv(mode,$thisseq)]} {
+		    set mhPriv(mode,$thisseq) public
+		}
+		if {$mhPriv(mode,$thisseq) == "public"} {
+		    if {![regexp {^ *$} $seqs($thisseq)]} {
+			if [regexp -- {-} $seqs($thisseq)] {
+			    set seq $seqs($thisseq)
+			} else {
+			    set seq [MhSeqMake $seqs($thisseq)]
+			}
+			puts $out "$thisseq: $seq"
+		    }
+		}
+	    }
+	    close $out
+	    Mh_Rename $filename.new $filename
+	    set mhPriv(seqmtime,$folder) [file mtime $filename]
+	} else {
+	    Exmh_Status "Couldn't write to $mhProfile(path)/$folder/$mhProfile(mh-sequences).new"
+	    set mhPriv(changed,private) 1
+	    foreach thisseq [array names seqs] {
+		set mhPriv(mode,$thisseq) "private"
+	    }
+	}
+    }
+    if {$mhPriv(changed,private) == 1} {
+	set mhPriv(privseq,$folder,$seq) [MhSeqExpand $folder $seqs($seq)]
+	set filename $mhProfile(context)
+	if {[catch {open $filename.new w} out] == 0} {
+	    Exmh_Debug Writing $filename
+	    puts $out [join $mhPriv(otherpriv) "\n"]
+	    foreach thisseq [array names seqs] {
+		if {[string compare $mhPriv(mode,$thisseq) "private"] == 0} {
+		    if {![regexp {^ *$} $seqs($thisseq)]} {
+			if [regexp -- {-} $seqs($thisseq)] {
+			    set seq $seqs($thisseq)
+			} else {
+			    set seq [MhSeqMake $seqs($thisseq)]
+			}
+			puts $out "atr-$thisseq-$mhProfile(path)/$folder: $seq"
+		    }
+		}
+	    }
+	    close $out
+	    Mh_Rename $filename.new $filename
+	    set mhPriv(privmtime) [file mtime $filename]
+	}
     }
 }
-
-proc MhSeq { how line msgs } {
-    if {![regexp {(.*: )(.*)} $line x prefix oldmsgs]} {
-	Exmh_Debug MhSeq $how regexp failed $line
-	return
-    }
-    Exmh_Debug MhSeq $how $line $msgs
-    set new [MhSeqExpand $msgs]
-    set old [MhSeqExpand $oldmsgs]
+proc MhSeq { folder seq how oldmsgids msgids } {
+    set new [MhSeqExpand $folder $msgids]
+    set old [MhSeqExpand $folder $oldmsgids]
     if {[string compare $how "add"] == 0} {
 	set merge [lsort -integer -increasing [concat $old $new]]
+	set seq [MhSeqMake $merge]
+	return $seq
     } elseif {[string compare $how "del"] == 0} {
 	set ix 0
 	set new [lsort -integer -increasing $new]
@@ -624,31 +690,31 @@ proc MhSeq { how line msgs } {
 		lappend merge $id
 	    }
 	}
-    } else {
+	return [MhSeqMake $merge]
+    } elseif {[string compare $how "replace"] == 0} {
 	# replace
-	Exmh_Debug $prefix $msgs
-	return "$prefix $msgs"
+	return [MhSeqMake $msgids]
+    } else {
+	return {}
     }
-    set seq [MhSeqMake $merge]
-    Exmh_Debug $prefix $seq
-    return "$prefix $seq"
 }
-proc MhSeqMake { msgs } {
-    Exmh_Debug MhSeqMake $msgs
+proc MhSeqMakeOld { msgs } {
     set result [lindex $msgs 0]
     set first $result
     set last $result
     set id {}
     foreach id [lrange $msgs 1 end] {
-	if {$id == $last + 1} {
-	    set last $id
-	} else {
-	    if {$last != $first} {
-		append result -$last
+	if {$id != $last} {
+	    if {$id == $last + 1} {
+		set last $id
+	    } else {
+		if {$last != $first} {
+		    append result -$last
+		}
+		set first $id
+		set last $id
+		append result " $first"
 	    }
-	    set first $id
-	    set last $id
-	    append result " $first"
 	}
     }
     if {$id == $last && [string length $msgs]} {
@@ -656,6 +722,34 @@ proc MhSeqMake { msgs } {
     }
     return $result
 }
+proc MhSeqMake { msgids } {
+    set result {}
+    set first {}
+    set last {}
+    foreach id $msgids {
+	if {$id == $last} {
+	    # Skipit
+	} elseif {($last != {}) && ($id == $last + 1)} {
+	    if {$first == {}} {
+		set first $last
+	    }
+	} else {
+	    if {$first != {}} {
+		lappend result "$first-$last"
+	    } elseif {$last != {}} {
+		lappend result $last
+	    } 
+	    set first {}
+	}
+	set last $id
+    }
+    if {$first != {}} {
+	lappend result "$first-$last"
+    } elseif {$last != {}} {
+	lappend result $last
+    }
+    return $result
+}
 
 proc Mh_Path { folder msg } {
     global mhProfile
@@ -671,10 +765,10 @@ proc Mh_Path { folder msg } {
 # tasks to run.  This causes a race between commit actions and background
 # inc/flist actions.
 
-proc Mh_Refile {srcFolder msgs folder} {
-    while {[llength $msgs] > 0} {
-	set chunk [lrange $msgs 0 19]
-	set msgs [lrange $msgs 20 end]
+proc Mh_Refile {srcFolder msgids folder} {
+    while {[llength $msgids] > 0} {
+	set chunk [lrange $msgids 0 19]
+	set msgids [lrange $msgids 20 end]
 	eval {MhExec refile} $chunk {-src +$srcFolder +$folder}
     }
 }
@@ -682,53 +776,52 @@ proc Mh_RefileFile {folder file} {
     Exmh_Debug exec refile -link -file $file +$folder
     eval {exec refile -link -file $file +$folder}
 }
-proc Mh_Copy {srcFolder msgs folder} {
-    while {[llength $msgs] > 0} {
-	set chunk [lrange $msgs 0 19]
-	set msgs [lrange $msgs 20 end]
+proc Mh_Copy {srcFolder msgids folder} {
+    while {[llength $msgids] > 0} {
+	set chunk [lrange $msgids 0 19]
+	set msgids [lrange $msgids 20 end]
 	eval {MhExec refile} $chunk {-link -src +$srcFolder +$folder}
     }
 }
-proc Mh_Rmm { folder msgs } {
-    while {[llength $msgs] > 0} {
-	set chunk [lrange $msgs 0 19]
-	set msgs [lrange $msgs 20 end]
+proc Mh_Rmm { folder msgids } {
+    while {[llength $msgids] > 0} {
+	set chunk [lrange $msgids 0 19]
+	set msgids [lrange $msgids 20 end]
 	eval {MhExec rmm +$folder} $chunk
     }
 }
-proc Mh_Send { msg } {
+proc Mh_Send { msgid } {
     global mhProfile
-
-    set path $mhProfile(path)/$mhProfile(draft-folder)/$msg
+    
+    set path $mhProfile(path)/$mhProfile(draft-folder)/$msgid
     set dst [Misc_PostProcess $path]
-
+    
     switch -- $mhProfile(sendType) {
 	"async" {
 	    MhExec $mhProfile(sendproc) -draftf +$mhProfile(draft-folder) \
-		    -draftm $dst -push -forward < /dev/null
+		-draftm $dst -push -forward < /dev/null
 	}
 	"wait" {
 	    MhExec $mhProfile(sendproc) -draftf +$mhProfile(draft-folder) \
-		    -draftm $dst < /dev/null
+		-draftm $dst < /dev/null
 	}
 	"xterm" {
 	    eval exec $mhProfile(xtermcmd) { \
-		-title "Sending $mhProfile(draft-folder)/$msg ..." \
+		-title "Sending $mhProfile(draft-folder)/$msgid ..." \
 		-e sh -c "$mhProfile(sendproc) -draftf +$mhProfile(draft-folder) -draftm $dst || whatnow -draftf +$mhProfile(draft-folder) -draftm $dst" &}
-
 	}
     }
-    if {$msg != $dst} {
+    if {$msgid != $dst} {
 	# In case we made a copy during post processing.
-	Mh_Rmm $mhProfile(draft-folder) $msg
+	Mh_Rmm $mhProfile(draft-folder) $msgid
     }
 }
-proc Mh_Whom { msg } {
+proc Mh_Whom { msgid } {
     global mhProfile
-    if {![regexp {^[0-9]+$} $msg]} {
-	MhExec whom $msg
+    if {![regexp {^[0-9]+$} $msgid]} {
+	MhExec whom $msgid
     } else {
-	MhExec whom -draftf +$mhProfile(draft-folder) -draftm $msg
+	MhExec whom -draftf +$mhProfile(draft-folder) -draftm $msgid
     }
 }
 
@@ -765,6 +858,13 @@ proc MhParseProfile {} {
     while {![eof $input]} {
 	set numBytes [gets $input line]
 	if {$numBytes > 0} {
+	    if {[regexp {^\s+(.*)$} $line foo other]} {
+		# handle continued lines
+		if {[info exists key]} {
+		    append mhProfile($key) " [string trim $other]"
+		}
+		continue
+	    }
 	    set parts [split $line :]
 	    set key [string tolower [lindex $parts 0]]
 	    set other [lindex $parts 1]
@@ -772,6 +872,7 @@ proc MhParseProfile {} {
 	    set mhProfile($key) $value
 	}
     }
+    close $input
     if {![info exists mhProfile(path)]} {
 	if {[info exists mhProfile(FAIL)]} {
 	    puts stderr "No Path entry in your [file tail $mhProfile(profile)] file."
@@ -803,7 +904,7 @@ proc MhParseProfile {} {
     if {![file exists $mhProfile(context)]} {
 	close [open $mhProfile(context) w]
     }
-
+    
     if {![info exists mhProfile(mh-sequences)]} {
 	set mhProfile(mh-sequences) .mh_sequences
     }
@@ -823,7 +924,7 @@ proc MhParseProfile {} {
 	set mhProfile(draft-folder) [string trim $mhProfile(draft-folder) +]
 	if {![file isdirectory $mhProfile(path)/$mhProfile(draft-folder)]} {
 	    Exmh_Status "Creating drafts folder"
-	    if {[catch {exec mkdir $mhProfile(path)/$mhProfile(draft-folder)} msg]} {
+	    if {[catch {exec mkdir $mhProfile(path)/$mhProfile(draft-folder)} msgid]} {
 		catch {
 		    puts stderr "Cannot create drafts folder $mhProfile(path)/$mhProfile(draft-folder)"
 		}
@@ -902,7 +1003,7 @@ Is it ok if Exmh sets up your MH environment for you?
 
     Widget_Frame .newuser rim Pad {top expand fill}
     .newuser.rim configure -bd 10
-
+    
     Widget_Frame .newuser.rim but Menubar {top fill}
     Widget_AddBut .newuser.rim.but yes "Yes" MhSetupNewUserInner
     Widget_AddBut .newuser.rim.but no "No, Exit" { destroy .newuser ; exit }
@@ -932,14 +1033,14 @@ and adding a draft-folder: entry
 to your [file tail $mhProfile(profile)].
 
 Should Exmh help you do that now?"
-
+    
     Widget_Frame .draft rim Pad {top expand fill}
     .draft.rim configure -bd 10
-
+    
     Widget_Label .draft.rim l {left} -text "Folder name: "
     Widget_Entry .draft.rim e {left fill}  -bg white
     .draft.rim.e insert 0 drafts
-
+    
     Widget_Frame .draft.rim but Menubar {top fill}
     Widget_AddBut .draft.rim.but yes "Yes" MhSetupDraftFolderInner
     Widget_AddBut .draft.rim.but no "Exit" { exit }
@@ -948,10 +1049,10 @@ Should Exmh help you do that now?"
 }
 proc MhSetupDraftFolderInner {} {
     global mhProfile
-
+    
     set dirname [.draft.rim.e get]
     set mhProfile(draft-folder) $dirname
-
+    
     set dir $mhProfile(path)/$mhProfile(draft-folder)
     if {![file isdirectory $dir]} {
 	if {[catch {
@@ -973,13 +1074,13 @@ proc MhSetupDraftFolderInner {} {
     puts $out "draft-folder: $dirname"
     Exmh_Status "draft-folder: $dirname"
     close $out
-
+    
     destroy .draft
 }
 proc MhSetupUnseenSequence {} {
     global mhProfile
     set mhProfile(unseen-sequence) unseen
-
+    
     if {[catch {open $mhProfile(profile) a} out]} {
 	Exmh_Status "Cannot open $mhProfile(profile): $out" error
 	unset mhProfile(unseen-sequence)
@@ -991,7 +1092,7 @@ proc MhSetupUnseenSequence {} {
 }
 proc MhSetMailDrops {} {
     global exdrops env mhProfile exdropMtime
-
+    
     global inc
     if {![regexp multi $inc(style)]} {
 	return
@@ -1002,7 +1103,7 @@ proc MhSetMailDrops {} {
     } else {
 	set name .xmhcheck
     }
-
+    
     if {[file exists $env(HOME)/$name]} then {
 	set mtime [file mtime $env(HOME)/$name]
 	if {[info exists exdropMtime]} {
@@ -1039,7 +1140,7 @@ proc MhSetMailDrops {} {
 		}
 		# Setup $unique as a unique identifier for this maildrop
 		# avoids clashes when you have 2 drops going to one folder
-	        if {$fields == 2} {
+		if {$fields == 2} {
 		    set u "local"
 		} 
 		set unique "$f-$d-$u"
@@ -1051,18 +1152,18 @@ proc MhSetMailDrops {} {
 	catch {puts stderr "Multidrop needs $name mapping file"}
     }
 }
-proc Mhn_DeleteOrig { msg } {
+proc Mhn_DeleteOrig { msgid } {
     global mhProfile
-    set path $mhProfile(path)/$mhProfile(draft-folder)/$mhProfile(delprefix)$msg
+    set path $mhProfile(path)/$mhProfile(draft-folder)/$mhProfile(delprefix)$msgid
     if {[file exists $path.orig]} {
 	Exmh_Debug Mhn_DeleteOrig deleting $path.orig
 	File_Delete $path.orig
     }
 }
 
-proc Mhn_RenameOrig { msg } {
+proc Mhn_RenameOrig { msgid } {
     global mhProfile
-    set path $mhProfile(path)/$mhProfile(draft-folder)/$mhProfile(delprefix)$msg
+    set path $mhProfile(path)/$mhProfile(draft-folder)/$mhProfile(delprefix)$msgid
     if {[file exists $path.orig]} {
 	Exmh_Debug Edit_Done moving $path.orig to $path
 	catch {Mh_Rename $path.orig $path}
@@ -1096,13 +1197,13 @@ proc Mh_Rename { old new } {
 proc Mh_FindFile { filename } {
     global mhProfile exmh
     if {[file exists [file join $mhProfile(path) $exmh(folder) $filename]]} {
-        return $exmh(folder)
+	return $exmh(folder)
     }
     set path $exmh(folder)
     while {[string compare [set path [file dirname $path]] "."] != 0} {
-        if {[file exists [file join $mhProfile(path) $path $filename]]} {
-            return $path
-        }
+	if {[file exists [file join $mhProfile(path) $path $filename]]} {
+	    return $path
+	}
     }
     # Not found until got to $mhProfile(path), return null string
     return ""
