@@ -522,6 +522,17 @@ proc MimeSetPartVars {descVar displayedPartVar tkw part partTag} {
 	    return;	# fastpath
 	}
     }
+    if {[string compare $mimeHdr($part,type) "application/octet-stream"] \
+     == 0} {
+	if [string length [set type [Mime_Magic \
+	 [Mime_GetUnencodedFile $part]]]] {
+	    set mimeHdr($part,hdr,content-description) $type
+	    regsub octet-stream $mimeHdr($part,hdr,content-type) \
+	     $type mimeHdr($part,hdr,content-type)
+	    set mimeHdr($part,type) $type
+	}
+    }
+
     MimeSetDisplayFlag $part
     global mimeContentId
     upvar $displayedPartVar displayedPart
@@ -544,17 +555,6 @@ proc MimeSetPartVars {descVar displayedPartVar tkw part partTag} {
 	}
     }
 
-    if {[string compare $mimeHdr($part,type) "application/octet-stream"] \
-     == 0} {
-	if [string length [set type [Mime_Magic \
-	 [Mime_GetUnencodedFile $part]]]] {
-	    set mimeHdr($part,hdr,content-description) $type
-	    regsub octet-stream $mimeHdr($part,hdr,content-type) \
-	     $type mimeHdr($part,hdr,content-type)
-	    set mimeHdr($part,type) $type
-	}
-    }
-
     foreach key [list $part,hdr,content-description \
 		      $part,param,name \
 		      $part=1,hdr,subject] {
@@ -603,7 +603,7 @@ proc MimeClose { fileIO } {
     }
 }
 proc MimeSetDisplayFlag {part} {
-    global mime mimeHdr msg
+    global mime mimeHdr msg uri
 
     # Flag to determine if we display the part or not.	We display it
     # if it's a multipart (other than multipart/parallel).  We display 
@@ -617,6 +617,9 @@ proc MimeSetDisplayFlag {part} {
 	} elseif [regexp {^image} $mimeHdr($part,type)] {
 	    set mimeHdr($part,display) $mime(showImage)
 	    return
+	} elseif [regexp {text/html} $mimeHdr($part,type)] {
+	    set mimeHdr($part,display) $uri(deferDisplaysInline)
+	    return
 	}
 	if [regexp {^text.*vcard} $mimeHdr($part,type)] {
 	    set mimeHdr($part,display) 0
@@ -839,7 +842,8 @@ proc Mime_ShowDefault {tkw part} {
 		       "Invoke menu with right button."
 	Mime_ShowMultipart $tkw $part
     } elseif {![catch {set viewer [MimeGetRule $part "" atrib]}] && 
-	[info exists atrib(copiousoutput)]} {
+	[info exists atrib(copiousoutput)] &&
+	![info exists atrib(needsterminal)]} {
 	if ![info exists mimeHdr($part,copiousOut)] {
 	    set newFile [Mime_TempFile $part]
 	    if [catch {exec sh -c $viewer > $newFile} err] {
@@ -1147,7 +1151,7 @@ proc MimeExtractMessage {tkw part} {
 
     Mh_RefileFile $exmh(folder) $mimeHdr($part,file)
     Scan_Folder $exmh(folder) 0
-    Exmh_Status "Extraction of part $part...done" blue
+    Exmh_Status "Extraction of part $part...done"
 }
 
 proc Mime_ShowMDN {tkw part} {
@@ -2787,9 +2791,15 @@ proc Mime_Magic { fileName } {
     global exmh env
     Exmh_Debug Mime_Magic $fileName
 
-    foreach m { "-m $exmh(userLibrary)/.magic" \
-     "-m $exmh(library)/local.magic" ""} {
-	if [catch {eval exec file $m $fileName} result] {
+    foreach m [list $exmh(userLibrary)/.magic \
+	     $exmh(library)/local.magic" "" ] {
+	# Avoid eval in this command to tolerate spaces in file names
+	set cmd {exec file}
+	if {[string length $m]} {
+	    lappend cmd -m $m
+	}
+	lappend cmd $fileName
+	if [catch $cmd result] {
 	    continue
 	}
 
