@@ -19,33 +19,12 @@ proc Msg_Init {} {
 proc Msg_Reset { numMsgs {folder {}} } {
     # Reset state after scanning a new folder
     global msg
-    set msg(seen) {}			;# list of seen messages
-    set msg(seenOld) {}			;# seen, then deleted or moved
     set msg(dpy)  {}			;# Currently displayed message
     set msg(id) [Mh_Cur $folder]	;# pick current message
     set msg(path) ""			;# File pathname of current message
     Buttons_Current [expr {$msg(id) != {}}]	;# Enable/disable buttons
     Ftoc_Reset $numMsgs $msg(id) $folder	;# Reset display
 }
-proc Msg_CheckPoint {} {
-    # push current MH state to disk
-    global exmh msg
-    if {$msg(id) != ""} {
-	set cur $msg(id)
-    } else {
-	set cur ""
-    }
-    if {$cur != ""} {
-	Mh_SetCur $exmh(folder) $cur
-    } else {
-	Mh_ClearCur $exmh(folder)
-    }
-    if {$msg(seen) != {}} {
-	Mh_MarkSeen $exmh(folder) $msg(seen)
-	set msg(seen) {}
-    }
-    set msg(seenOld) {}
-}
 proc Msg_Pick { line {show show} } {
     # Select a message
     global exwin msg
@@ -53,14 +32,24 @@ proc Msg_Pick { line {show show} } {
     set msgNum [Ftoc_MsgNumber $line]
     if {$msgNum != {} && $msgNum != 0} {
 	Ftoc_RangeUnHighlight
-	Msg_Change $msgNum $show $line
+	Msg_Change $msgNum $show
     } else {
 	Msg_ClearCurrent
     }
 }
-
-proc Msg_ShowCurrent { {show show} } {
-    global msg
+proc Msg_Show { {seq cur} {show show} } {
+    global exmh msg
+    Exmh_Debug Msg_Show $seq $show
+    if {$seq != "cur"} {
+	foreach id [Seq_Msgs $exmh(folder) $seq] {
+	    if {![Ftoc_Marked $id]} {
+		Msg_Change $id $show
+		return 1
+	    }
+	}
+	# If we can't find a message in $seq, then fall through to do
+	# what we would have done if we were going to 'cur'.
+    } 
     if {$msg(id) != {}} {
 	set msg(dpy) {}	;# force redisplay
 	Msg_Change $msg(id) $show
@@ -70,23 +59,16 @@ proc Msg_ShowCurrent { {show show} } {
 	Ftoc_Yview end
 	return 0
     }
-}
-proc Msg_ShowUnseen { {show show} } {
-    global exmh
-    foreach id [Flist_UnseenMsgs $exmh(folder)] {
-	if {![Ftoc_Marked $id]} {
-	    Msg_Change $id show
-	    return 1
-	}
-    }
-    Msg_ClearCurrent
     return 0
 }
+proc Msg_ShowCurrent {} {
+    Msg_Show cur
+}
 proc Msg_ClearCurrent { } {
     global msg exmh
     set msg(id) {}		;# Clear current message
     set msg(dpy) {}		;# and currently displayed message
-    Mh_ClearCur $exmh(folder)
+    Seq_Forget $exmh(folder) cur
     MsgClear
     Buttons_Current 0
     Uri_ClearCurrent
@@ -102,8 +84,8 @@ proc MsgClear {} {
     catch {destroy $exwin(mopButtons).list}
 }
 proc Msg_ShowSomething {} {
-    global exmh msg
-    set order {unseen cur}
+    global exmh msg mhProfile
+    set order [list $mhProfile(unseen-sequence) cur]
     foreach pick $order {
 	if {[catch {MhExec pick +$exmh(folder) $pick} tmp] == 0} then {
 	    Msg_Change [lindex $tmp 0] show
@@ -114,40 +96,28 @@ proc Msg_ShowSomething {} {
     global ftoc
     Msg_Pick $ftoc(numMsgs) show
 }
-proc Msg_ShowWhat { {what last} {show show} } {
-    if {$what == {}} {
-	Msg_ClearCurrent
-	return 0
-    } else {
-	Msg_Change {} $show $what
-	return 1
-    }
-}
 proc Msg_First { {show noshow} } {
-    Msg_Change {} $show first
+    Msg_Change [Ftoc_MsgNumber 1] $show
 }
 
 proc Msg_Last { {show noshow} } {
-    Msg_Change {} $show last
+    global ftoc
+    Msg_Change [Ftoc_MsgNumber $ftoc(numMsgs)] $show
 }
 
-proc Msg_Change {msgid {show show} {line {}} } {
-    Exmh_Debug Msg_Change id=$msgid line=$line
-    Exmh_Debug Msg_Change [time [list MsgChange $msgid $show $line]]
+proc Msg_Change {msgid {show show} } {
+    Exmh_Debug Msg_Change id=$msgid
+    Exmh_Debug Msg_Change [time [list MsgChange $msgid $show]]
 }
-proc MsgChange {msgid {show show} {line {}} } {
+proc MsgChange {msgid {show show}} {
     global exmh exwin msg mhProfile
-
-    if {$msgid != {}} {
-	# Allow null msgid from Msg_ShowWhat, which supplies line instead
-	if {$msgid < 0}  return
-    } else {
-        # line null too, try using first in folder
-        if {[string length $line] == 0} { set line 1 }
-	set msgid [Ftoc_MsgNumber [Ftoc_FindMsg $msgid $line]]
-    }
+    
     Ftoc_ClearCurrent
-    if {! [Ftoc_Change $msgid $line $show]} {
+    set oldcur [Seq_Msgs $exmh(folder) cur]
+    Mh_SetCur $exmh(folder) $msgid
+    Ftoc_ShowSequence cur [concat $oldcur $msgid]
+    set lineno [Ftoc_FindMsg $msgid]
+    if {! [Ftoc_Change $lineno $show]} {
 	Exmh_Status "Cannot find msg $msgid - Rescan?"
     } else {
 	if {$msg(id) == {}} {
@@ -157,44 +127,22 @@ proc MsgChange {msgid {show show} {line {}} } {
 	set msg(path) $mhProfile(path)/$exmh(folder)/$msg(id)
 	if {$show == "show"} {
 	    MsgShow $msgid
-	} else {
+	    Seq_Del $exmh(folder) $mhProfile(unseen-sequence) $msgid
+	} elseif {$show != "skipdisplay"} {
 	    MsgClear
 	}
-	if {$line != {}} {
-	    Ftoc_MoveFeedback $msgid $line
+	if {$lineno != {}} {
+	    Ftoc_MoveFeedback $msgid
 	}
     }
-    Folder_CheckPointShared
 }
 
 proc MsgSeen { msgid } {
     # Suppress duplicates or else mark does the wrong thing.
-    global msg
-    if {[lsearch $msg(seen) $msgid] < 0} {
-	lappend msg(seen) $msgid
-    }
-    Flist_MsgSeen $msgid
+    global msg exmh mhProfile
+    Seq_Del $exmh(folder) $mhProfile(unseen-sequence) $msgid
     Flag_MsgSeen
 }
-proc Msg_UnSeen { msgid } {
-    # We nuke deleted and moved messages from the seen list because
-    # marking them confuses MH.  However, we still need to remember
-    # them to properly maintain our unseen state in the presense of
-    # background Flist_FindUnseen calls.  Hence msg(seenOld)
-    global msg
-    set ix [lsearch $msg(seen) $msgid]
-    if {$ix >= 0} {
-	set msg(seen) [lreplace $msg(seen) $ix $ix]
-	if {[lsearch $msg(seenOld) $msgid] < 0} {
-	    lappend msg(seenOld) $msgid
-	}
-    }
-}
-proc Msg_Seen {} {
-    global msg
-    return [concat $msg(seenOld) $msg(seen)]
-}
-
 
 # Message operations.
 # These take two forms of arguments.  The original form is a single
@@ -210,7 +158,7 @@ proc Msg_Compose { args } {
     if {[string compare [info command $args] $args] == 0} {
 	# Old interface with hook procedure
 	if [catch {$args} err] {			;# Setup draft msg
-	    Exmh_Status "$args: $err" purple
+	    Exmh_Status "$args: $err" error
 	    return
 	}
     } else {
@@ -330,7 +278,7 @@ proc MsgComp {args} {
 
 proc Msg_ReplyAll { } {
     global nmh
-
+    
     if {$nmh == 1} {
 	Msg_Reply -group
     } else {
@@ -343,19 +291,19 @@ proc Msg_Reply { args } {
     if {[string length $args] == 0} {
 	set args Mh_ReplySetup
     }
-
+    
     if [MsgOk $msg(id) m] {
 	Quote_MakeFile $exmh(folder) $m
 	set edit 1
 	if {[string compare [info command $args] $args] == 0} {
 	    # Old interface with hook procedure
 	    if [catch {$args $exmh(folder) $m} err] {	;# Setup draft msg
-		Exmh_Status "${args}: $err" purple
+		Exmh_Status "${args}: $err" error
 		Quote_Cleanup
 		return
 	    }
 	} else {
-	    Exmh_Status "repl $args" purple
+	    Exmh_Status "repl $args" error
 	    if [catch {
 		set ix [lsearch $args -noedit]
 		if {$ix >= 0} {
@@ -364,16 +312,16 @@ proc Msg_Reply { args } {
 		}
 		set ix [lsearch $args -form]
 		if {$ix < 0} {
-                    set path [Mh_FindFile "replcomps"]
+		    set path [Mh_FindFile "replcomps"]
 		    if {0 != [string length $path]} {
 			lappend args -form $path/replcomps
-			Exmh_Status "repl $args" purple
+			Exmh_Status "repl $args" error
 		    }
 		}
 		eval {MhExec repl +$exmh(folder) $m -nowhatnowproc} $args
 		eval {MhAnnoSetup $exmh(folder) $m repl} $args
 	    } err] {	;# Setup draft msg
-		Exmh_Status "repl: $err" purple
+		Exmh_Status "repl: $err" error
 		Quote_Cleanup				;# Nuke @ link
 		return
 	    }
@@ -383,7 +331,7 @@ proc Msg_Reply { args } {
 	} else {
 	    Edit_Done send				;# Just send it
 	}
-   }
+    }
 }
 
 proc Msg_Forward { args } {
@@ -391,16 +339,9 @@ proc Msg_Forward { args } {
     if {[string length $args] == 0} {
 	set args Mh_ForwSetup
     }
-
-    set ids {}
-    Ftoc_Iterate line {
-	set msgid [Ftoc_MsgNumber $line]
-	if {$msgid != {}} {
-	    lappend ids $msgid
-	}
-    }
-    if {[llength $ids] > 0} {
-	global mhProfile
+    
+    set msgids [Ftoc_CurMsgs]
+    if {[llength $msgids] > 0} {
 	set mime 0
 	if [info exists mhProfile(forw)] {
 	    if {[lsearch $mhProfile(forw) -mime] >= 0} {
@@ -409,12 +350,12 @@ proc Msg_Forward { args } {
 	}
 	if {[string compare [info command $args] $args] == 0} {
 	    # Old interface with hook procedure
-	    if [catch {$args $exmh(folder) $ids} err] {	;# Setup draft msg
-		Exmh_Status "${args}: $err" purple
+	    if [catch {$args $exmh(folder) $msgids} err] {	;# Setup draft msg
+		Exmh_Status "${args}: $err" error
 		return
 	    }
 	}  else {
-	    Exmh_Status "forw +$exmh(folder) $ids $args"
+	    Exmh_Status "forw +$exmh(folder) $msgids $args"
 	    if [catch {
 		if {[lsearch $args -mime] >= 0} {
 		    set mime 1
@@ -423,16 +364,16 @@ proc Msg_Forward { args } {
 		if {$ix < 0} {
 		    if [file exists $mhProfile(path)/$exmh(folder)/forwcomps] {
 			lappend args -form $exmh(folder)/forwcomps
-			Exmh_Status "forw +$exmh(folder) $ids $args"
+			Exmh_Status "forw +$exmh(folder) $msgids $args"
 		    }
 		}
-		eval {MhExec forw +$exmh(folder)} $ids -nowhatnowproc $args
-		eval {MhAnnoSetup $exmh(folder) $ids forw} $args
+		eval {MhExec forw +$exmh(folder)} $msgids -nowhatnowproc $args
+		eval {MhAnnoSetup $exmh(folder) $msgids forw} $args
 		if {$mhProfile(forwtweak)} {
-		    Mh_Forw_MungeSubj $exmh(folder) $ids
+		    Mh_Forw_MungeSubj $exmh(folder) $msgids
 		}
 	    } err] {
-		Exmh_Status "forw: $err" purple
+		Exmh_Status "forw: $err" error
 		return
 	    }
 	}
@@ -442,7 +383,7 @@ proc Msg_Forward { args } {
 	if {$mime} {set sedit(mhnDefault) 1}
 	Edit_Draft					;# Run the editor
 	set sedit(mhnDefault) $old
-   }
+    }
 }
 
 proc Msg_Dist { args } {
@@ -450,12 +391,12 @@ proc Msg_Dist { args } {
     if {[string length $args] == 0} {
 	set args Mh_DistSetup
     }
-
+    
     if [MsgOk $msg(id) m] {
 	if {[string compare [info command $args] $args] == 0} {
 	    # Old interface with hook procedure
 	    if [catch {$args $exmh(folder) $m} err] {   ;# Setup draft msg
-		Exmh_Status "${args}: $err" purple
+		Exmh_Status "${args}: $err" error
 		return
 	    }
 	}  else {
@@ -464,31 +405,30 @@ proc Msg_Dist { args } {
 		eval {MhExec dist +$exmh(folder) $m} -nowhatnowproc $args
 		eval {MhAnnoSetup $exmh(folder) $m dist} $args
 	    } err] {
-		Exmh_Status "dist: $err" purple
+		Exmh_Status "dist: $err" error
 		return
 	    }
 	}
 	Edit_Draft                                  ;# Run the editor
     }
 }
- 
+
 proc MsgOk { number msgvar } {
     upvar $msgvar msg
     if {$number != ""} {
 	set msg $number
 	return 1
     } else {
-	Exmh_Status "No valid message number" red
+	Exmh_Status "No valid message number" warning
 	return 0
     }
 }
 
 proc Msg_Remove { {rmProc Ftoc_RemoveMark} {show show} } {
-    Exmh_Debug Msg_Remove $rmProc
-    Ftoc_Iterate line {
-	set msgid [Ftoc_MsgNumber $line]
-	Exmh_Debug Msg_Remove l=$line m=$msgid
-	$rmProc $line $msgid
+    Exmh_Debug Msg_Remove $rmProc $show
+    Ftoc_Iterate lineno {
+	Exmh_Debug Msg_Remove l=$lineno
+	$rmProc $lineno
     }
     if {[Ftoc_PickSize] == 1} {
 	Ftoc_NextImplied $show
@@ -499,24 +439,22 @@ proc Msg_RemoveNoshow { {rmProc Ftoc_RemoveMark} } {
 }
 proc Msg_RemoveById { msgid {rmProc Ftoc_Delete} } {
     global msg
-    set line [Ftoc_FindMsg $msgid]
-    $rmProc $line $msgid
-    Msg_UnSeen $msgid
+    set lineno [Ftoc_FindMsg $msgid]
+    $rmProc $lineno
     if {$msg(id) == $msgid} {
 	Msg_ClearCurrent
     }
 }
 proc Msg_Move { {moveProc Ftoc_MoveMark} {advance 1} {show show} } {
     global exmh fdisp
-
+    
     if {$exmh(target) == ""} {
-	Exmh_Status "Must first click button $fdisp(tarbutton) on folder label to pick destination" purple
+	Exmh_Status "Must first click button $fdisp(tarbutton) on folder label to pick destination" error
 	return
     }
     if { $exmh(target) != $exmh(folder)} then {
-	Ftoc_Iterate line {
-	    set msgid [Ftoc_MsgNumber $line]
-	    $moveProc $line $msgid
+	Ftoc_Iterate lineno {
+	    $moveProc $lineno
 	}
 	Exmh_Status "=> $exmh(target)"
 	if {[Ftoc_Advance $advance] && ([Ftoc_PickSize] == 1)} {
@@ -532,12 +470,12 @@ proc Msg_MoveNoshow { {moveProc Ftoc_MoveMark} } {
 proc Msg_Clip { {folder {}}  {id {}} } {
     # "Tear off" a message into a top-level text widget
     global mhProfile exmh msg exwin
-
+    
     if {$folder == {}} {set folder $exmh(folder)}
     if {$id     == {}} {set id     $msg(id)}
-
+    
     if {$id == {}} {
-	Exmh_Status "Select a message to clip first" red
+	Exmh_Status "Select a message to clip first" warning
 	return
     }
     if ![info exists msg(tearid)] {
@@ -546,7 +484,7 @@ proc Msg_Clip { {folder {}}  {id {}} } {
 	incr msg(tearid)
     }
     set self [Widget_Toplevel .tear$msg(tearid) "$folder $id" Clip]
-
+    
     Widget_Frame $self but Menubar {top fill}
     Widget_AddBut $self.but quit "Dismiss" [list destroy $self]
     Widget_Label $self.but label {left fill} -text $folder/$id
@@ -554,13 +492,13 @@ proc Msg_Clip { {folder {}}  {id {}} } {
     set t [Widget_Text $self $exwin(mtextLines) -cursor $cursor -setgrid true]
     Msg_Setup $t
     if [MsgShowInText $t $mhProfile(path)/$folder/$id] {
-        foreach cmd [info commands Hook_MsgClip*] {
+	foreach cmd [info commands Hook_MsgClip*] {
             if [catch {$cmd $mhProfile(path)/$folder/$id $t} err] {
-                SeditMsg $t "$cmd $err"
-            }
+		SeditMsg $t "$cmd $err"
+	    }
         }
     }
-
+    
 }
 proc Msg_FindMatch {L string} {
     global exwin
@@ -568,42 +506,42 @@ proc Msg_FindMatch {L string} {
 }
 proc Msg_BurstDigest {} {
     global msg exmh mhProfile
-
+    
     if {$msg(id) == {}} {
-	Exmh_Status "No message selected to burst" purple
+	Exmh_Status "No message selected to burst" error
 	return
     }
     if {[Ftoc_Changes "Burst Digest"] != 0} {
 	# Pending changes and no autoCommit
 	return
     }
-
-    Exmh_Status "Bursting message $msg(id) in $exmh(folder)..." blue
-
+    
+    Exmh_Status "Bursting message $msg(id) in $exmh(folder)..."
+    
     # burst the digest; catch the output
     if [catch { MhExec burst -verbose $msg(id) +$exmh(folder)} out] {
 	Exmh_Status "Error bursting digest: $out"
     } else {
 	# burst OK, split up the output
-	set allids {}
+	set allmsgids {}
 	foreach line [ split $out \n] {
-	    #extract the new message number and save in $allids
+	    #extract the new message number and save in $allmsgids
 	    if [regexp {of digest .* becomes message ([0-9]+)} $line match msgid] {
-		lappend allids $msgid
+		lappend allmsgids $msgid
 	    }
 	}
-	set allids [lsort -increasing -integer $allids]
+	set allmsgids [lsort -increasing -integer $allmsgids]
 	# mark new messages as unread
-	Exmh_Debug burst created msgs $allids
-	if {$allids != {}} {
-	    eval { MhExec mark +$exmh(folder) -sequence $mhProfile(unseen-sequence) } $allids
+	Exmh_Debug burst created msgs $allmsgids
+	if {$allmsgids != {}} {
+	    eval { MhExec mark +$exmh(folder) -sequence $mhProfile(unseen-sequence) } $allmsgids
 	}
 	# rescan to pick them up, make sure Commit is done.
 	Background_Wait
-	Exmh_Status "Bursting message $msg(id) in $exmh(folder)...done" blue
+	Exmh_Status "Bursting message $msg(id) in $exmh(folder)...done"
 	Scan_FolderUpdate $exmh(folder)
-	if {$allids != {}} {
-	    Msg_Change [lindex $allids 0]
+	if {$allmsgids != {}} {
+	    Msg_Change [lindex $allmsgids 0]
 	} else {
 	    Msg_ClearCurrent
 	}
@@ -612,11 +550,10 @@ proc Msg_BurstDigest {} {
 proc Msg_Save {} {
     global exmh mhProfile
     set files {}
-    Ftoc_Iterate line {
-	set msgid [Ftoc_MsgNumber $line]
+    Ftoc_MsgIterate msgid {
 	lappend files $mhProfile(path)/$exmh(folder)/$msgid
     }
-
+    
     set name [FSBox "Select file to create/append to:" ]
     if {$name != {}} {
 	set exists [file exists $name]
@@ -665,7 +602,7 @@ proc Msg_Edit {} {
 }
 
 proc Msg_UUdecode {} {
-    global exmh msg mhProfile
+    global msg
     set name [FSBox "Select file to decode into:" ]
     if {$name != {}} {
 	Mime_Uudecode $msg(path) $name
@@ -674,17 +611,21 @@ proc Msg_UUdecode {} {
     }
 }
 
-proc Msg_MarkUnseen {} {
-    global exmh
-    Msg_CheckPoint
-    Ftoc_Iterate line {
-	set msgid [Ftoc_MsgNumber $line]
-	Mh_MarkUnseen $exmh(folder) $msgid
+proc Msg_Mark {seq} {
+    global exmh mhProfile
+    set msgids [Ftoc_CurMsgs]
+    Seq_Add $exmh(folder) $seq $msgids
+    if {$seq == $mhProfile(unseen-sequence)} {
+	Msg_ClearCurrent
+	Ftoc_ClearCurrent
     }
-    Msg_ClearCurrent
-    Ftoc_ClearCurrent
-    Flist_ForgetUnseen $exmh(folder)
-    Ftoc_ShowUnseen $exmh(folder)
+    Ftoc_ShowSequence $seq $msgids
+}
+proc Msg_UnMark {seq} {
+    global exmh mhProfile
+    set msgids [Ftoc_CurMsgs]
+    Seq_Del $exmh(folder) $seq $msgids
+    Ftoc_ShowSequence $seq $msgids
 }
 
 proc Msg_ReplyHelp {} {
