@@ -6,6 +6,38 @@
 # 
 
 # $Log: pgpExec.tcl,v $
+# Revision 1.19  2002/07/16 01:27:54  sysphrog
+# Fixing problems with PGP sign+encrypt, gnupg 1.0.7 support
+#
+# Revision 1.18  2002/05/01 02:24:07  welch
+# A whole collection of patches.  If marked with ** then I've lost
+# track of who gave them to me and I apologize for that:
+# exmh-strip.MASTER: added pref initialization to quiet errors caused
+#   by changes elsewhere in the main body of exmh
+# install.tcl: fixed errors that occur when you try to display a
+#   dialog box (e.g., the Verify window) that is already displayed
+# lib/addr.tcl: a new set of options for configuring LDAP (Mark Bergman)
+# lib/extrasInit.tcl: help text updates about the uquoteAdd resource (**)
+# lib/faces.tcl: fix for space-in-pathname problem (**)
+# lib/fcache.tcl: New Feature! display the count of unseen messages
+#   in the folder cache. (Paul Menage)
+# lib/html_get_http.tcl: trap errors from bad http: links
+# lib/inc.tcl: tweaked feedback about inc'ed messages to do case-insensitive
+#   grep for Subject: (**)
+# lib/mime.tcl: for for space-in-pathname problem
+# lib/pgpExec.tcl: eliminated Exmh_Debug message that could dump out
+#   a massive keyring to the log, taking many many seconds (**)
+# lib/unseenwin.tcl: fix to tolerate space-in-folder-name (I think) (**)
+#
+# Revision 1.17  2001/12/08 00:39:52  kchrist
+# Fixed "GPG silently ignores untrusted keys during encryption" bug.
+# Thanks to Ben Escoto.
+#
+# Revision 1.16  2001/12/06 16:39:13  kchrist
+# Exmh can now parse the GnuPG options file and identify the
+# "default-key" (same as "myname" in PGP).  Added "--status-fd 2" to
+# args_decrypt so that the output can be parsed with Pgp_InterpretOutput.
+#
 # Revision 1.15  2000/09/21 15:06:44  valdis
 # Catch PGP stderr so 'Get key' and 'Generate Key' work...
 #
@@ -295,7 +327,9 @@ proc Pgp_Exec_KeyList { v pattern keyringtype } {
     regsub -all [set pgp($v,pat_splitKeys)] $keylist \x81 keylist
     set keylist [split $keylist \x81]
 
-    Exmh_Debug "<Pgp_Exec_Keylist>: Splitted keylist: $keylist"
+    # This print statement converts keylist from a Tcl list to
+    # a string. For really big keylists, this is reportedly very expensive
+    # Exmh_Debug "<Pgp_Exec_Keylist>: Splitted keylist: $keylist"
 
     # Match out interesting keys
     set keypattern [set pgp($v,pat_key$keyringtype)]
@@ -365,8 +399,14 @@ proc Pgp_Exec_ParseConfigTxt { v file } {
     if [catch {open $file r} in] {
 	return
     }
+    if {$v != "gpg"} {
+	set pat "^\[ \t]*(\[a-z]+)\[ \t]*=(\[^#]*)"
+    } else {
+	# GnuPG uses space as separator and options may have dashes
+	set pat "^\[ \t]*(\[a-z-]+)\[ \t]*(\[^#]*)"
+    }
     for {set len [gets $in line]} {$len >= 0} {set len [gets $in line]} {
-	if [regexp -nocase "^\[ \t]*(\[a-z]+)\[ \t]*=(\[^#]*)" $line {} option value] {
+	if [regexp -nocase $pat $line {} option value] {
 	    set pgp($v,config,[string tolower $option]) [string trim $value " \t\""]
 	}
     }
@@ -387,7 +427,9 @@ proc Pgp_Exec_Encrypt { v in out tokeys } {
         # pgp refuses to generate an encrypted message
         # if a key was untrusted
         # interactively proceed
+        catch {file delete $out}
         Pgp_Exec_Interactive $v encrypt [subst [set pgp($v,args_encrypt)]] output
+        Pgp_Exec_CheckSuccess $v $out $output "encrypted text"
     }
 }
 
@@ -402,7 +444,9 @@ proc Pgp_Exec_EncryptSign { v in out sigkey tokeys } {
         # pgp refuses to generate an encrypted/signed message
         # if a key was untrusted
         # interactively proceed
+        catch {file delete $out}
         Pgp_Exec $v encrypt [subst [set pgp($v,args_encryptSign)]] output $sigkey 1
+        Pgp_Exec_CheckSuccess $v $out $output "signed and encrypted text"
     }
 }
  
@@ -431,17 +475,29 @@ proc Pgp_Exec_CheckSuccess {v out output object} {
     if {![file exists $out] && [file exists "$out.asc"]} {
 	Mh_Rename "$out.asc" $out
     }
-    if {![file exists $out]} {
-        # pgp5 refuses to generate ciphertext in batchmode if tokey is untrusted
-        if {[regexp [set pgp($v,pat_Untrusted)] $output]} {
+
+    if {$v != "gpg"} {
+    	# pgp5 refuses to generate ciphertext in batchmode 
+	# if tokey is untrusted
+    	if {![file exists $out]} {
+	    if {[regexp [set pgp($v,pat_Untrusted)] $output]} {
+		return 1
+    	    } else {
+	    	error "[set pgp($v,fullName)] refused to generate the ${object}:\n$output"
+    	    }
+    	} else {
+	    return 0
+    	}
+    } else {
+        # GnuPG will also not encrypt to a key if it is untrusted but if
+        # any of the encryption keys are trusted a file will be generated
+        if {[regexp "^(.*\n)*gpg:.*no (info|indication)" $output]} {
             return 1
         } else {
-	    error "[set pgp($v,fullName)] refused to generate the ${object}:\n$output"
+            return 0
         }
-    } else {
-	return 0
     }
-}    
+}
 
 
 #################
@@ -765,6 +821,11 @@ proc Pgp_Exec_Init {} {
                 set pgp($v,secring) {}
             }
             set pgp($v,privatekeys) [Pgp_Exec_KeyList $v $pgp($v,ownPattern) Sec]
+	    # GnuPG uses default-key for what PGP uses myname
+	    if {![info exists pgp($v,config,myname] && \
+		    [info exists pgp($v,config,default-key)]} {
+		    set pgp($v,config,myname) $pgp($v,config,default-key)
+	    }
             #
             if [info exists pgp($v,config,myname)] {
                 set myname [string tolower [set pgp($v,config,myname)]]
