@@ -13,24 +13,25 @@
 
 #### Display folder contents
 
-proc Scan_Folder {F {adjustDisplay 1}} {
-    Exmh_Debug Scan_Folder [time [list ScanFolder $F $adjustDisplay]]
+proc Scan_Folder {folder {adjustDisplay 1}} {
+    Exmh_Debug Scan_Folder $folder $adjustDisplay
+    Exmh_Debug Scan_Folder [time [list ScanFolder $folder $adjustDisplay]]
 }
-proc ScanFolder {F adjustDisplay} {
+proc ScanFolder {folder adjustDisplay} {
     global mhProfile flist ftoc exwin exmh
 
-    if {[string compare $F $ftoc(folder)] == 0} {
-	Exmh_Debug Updating $F
+    if {[string compare $folder $ftoc(folder)] == 0} {
+	Exmh_Debug Updating $folder
 	set update 1	;# Need to check for new messages.
-	set sameF 1	;# Same folder as before
+	set samefolder 1	;# Same folder as before
 	ScanAddLineInit
     } else {
-	set sameF 0
-	set cacheFile $mhProfile(path)/$F/.xmhcache
+	set samefolder 0
+	set cacheFile $mhProfile(path)/$folder/.xmhcache
 	if [catch {open $cacheFile} input] {
 	    # No cache, scan last N messages
-	    Exmh_Status "Limit scan $F last:$ftoc(scanSize) - Rescan?" warn
-	    set input  [open "|$mhProfile(scan-proc) [list +$F] \
+	    Exmh_Status "Limit scan $folder last:$ftoc(scanSize) - Rescan?" warn
+	    set input  [open "|$mhProfile(scan-proc) [list +$folder] \
 		    last:$ftoc(scanSize) -width $ftoc(scanWidth)"]
 	    set ftoc(displayDirty) 1
 	    set update 0
@@ -39,7 +40,7 @@ proc ScanFolder {F adjustDisplay} {
 	    set ftoc(displayDirty) 0
 	    set update 1	;# Need to check for new messages.
 	}
-	ScanAddLineReset $F
+	ScanAddLineReset $folder
 	ScanAddLines [read $input]
 	catch {close $input}
     }
@@ -47,104 +48,103 @@ proc ScanFolder {F adjustDisplay} {
     if {$update} {
 	# Add new messages to cached information
 	# Scan last message (again) plus any new messages
-	if {! $sameF} {
-	    Ftoc_Reset [Widget_TextEnd $exwin(ftext)] {} $F
+	if {! $samefolder} {
+	    Ftoc_Reset [Widget_TextEnd $exwin(ftext)] {} $folder
 	}
-	set id [Ftoc_MsgNumber [Ftoc_FindMsg {} last]]
-	if [catch {
-	    Exmh_Debug Scanning new messages
-	    set input [open "|$mhProfile(scan-proc) [list +$F] \
-		    $id-last -width $ftoc(scanWidth)"]
-	    set check [gets $input]
-	    set new [read $input]
-	    close $input
-	} err] {
-	    # The last message no longer exists
-	    Exmh_Debug No last msg $id: $err
-        } else {
-	    set id2 [Ftoc_MsgNumberRaw $check]
-	    if {$id2 == $id} {
-		# Last message still matches: add the new lines
-		ScanAddLines $new
-		set ftoc(displayDirty) 1
-		set update 0	;# OK
+	if {$ftoc(numMsgs) > 0} {
+	    set msgid [Ftoc_MsgNumber $ftoc(numMsgs)]
+	    if [catch {
+		Exmh_Debug Scanning new messages $msgid-last
+		set input [open "|$mhProfile(scan-proc) [list +$folder] \
+		    $msgid-last -width $ftoc(scanWidth)"]
+		set check [gets $input]
+		set new [read $input]
+		close $input
+		Exmh_Debug Scan complete
+	    } err] {
+		# The last message no longer exists
+		Exmh_Debug No last msg $msgid: $err
 	    } else {
-		Exmh_Debug "My last $id != $id2"
+		set msgid2 [Ftoc_MsgNumberRaw $check]
+		if {$msgid2 == $msgid} {
+		    # Last message still matches: add the new lines
+		    ScanAddLines $new
+		    set ftoc(displayDirty) 1
+		    set update 0	;# OK
+		} else {
+		    Exmh_Debug "My last $msgid != $msgid2"
+		}
 	    }
 	}
 	if {$update} {
 	    # Something went wrong: rescan
 	    if {[Ftoc_Changes "Scan Update Failed"] == 0} {
-	    Exmh_Status "scan +$F last:$ftoc(scanSize)"
+	    Exmh_Status "scan +$folder last:$ftoc(scanSize)"
 		Background_Wait
-		set input  [open "|$mhProfile(scan-proc) [list +$F] \
+		set input  [open "|$mhProfile(scan-proc) [list +$folder] \
 			last:$ftoc(scanSize) -width $ftoc(scanWidth)"]
 		set ftoc(displayDirty) 1
-		ScanAddLineReset $F
+		ScanAddLineReset $folder
 		ScanAddLines [read $input]
 		catch {close $input}
 	    }
 	}
     }
-    ScanAddLineCleanup $F
-    if {! $sameF} {
-	Msg_Reset [Widget_TextEnd $exwin(ftext)] $F
+    ScanAddLineCleanup $folder
+    if {! $samefolder} {
+	Msg_Reset [Widget_TextEnd $exwin(ftext)] $folder
     } else {
-	Ftoc_Update [Widget_TextEnd $exwin(ftext)] $F
+	Ftoc_Update [Widget_TextEnd $exwin(ftext)]
     }
     set ftoc(displayValid) 1
     if {$adjustDisplay} {
 	Ftoc_Yview end
     }
-    if {$update} {
-	Flist_ForgetUnseen $F
-    }
-    Ftoc_ShowUnseen $F
     return
 }
-proc Scan_FolderForce {{F ""}} {
+proc Scan_FolderForce {{folder ""}} {
     global exmh mhProfile ftoc
-    if {$F == ""} {
-	set F $exmh(folder)
+    if {$folder == ""} {
+	set folder $exmh(folder)
     }
-    set cacheFile $mhProfile(path)/$F/.xmhcache
-    if {$F == ""} {
-	Exmh_Status "No current folder" red
-    } elseif {$F != $exmh(folder)} {
+    set cacheFile $mhProfile(path)/$folder/.xmhcache
+    if {$folder == ""} {
+	Exmh_Status "No current folder" warning
+    } elseif {$folder != $exmh(folder)} {
 	# If we aren't currently viewing the folder, just delete
 	# the cache file and we'll take care of this later
 	Exmh_Debug "Clearing $cacheFile"
 	file delete $cacheFile
     } elseif {! [Ftoc_Changes Rescan]} {
 	Background_Wait
-	Label_Folder $F
-	Exmh_Status "rescanning $F ..."
-	Scan_IO $F [open "|$mhProfile(scan-proc) [list +$F] \
+	Label_Folder $folder
+	Exmh_Status "rescanning $folder ..."
+	Scan_IO $folder [open "|$mhProfile(scan-proc) [list +$folder] \
 		-width $ftoc(scanWidth)"]
 	set ftoc(displayValid) 1
 	set ftoc(displayDirty) 1
 	Ftoc_Yview end
-	Flist_ForgetUnseen $F
-	Ftoc_ShowUnseen $F
+	Seq_Msgs $folder $mhProfile(unseen-sequence)
+	Ftoc_ShowSequences
 	Exmh_Status ok
     }
 }
-proc Scan_FolderUpdate { f } {
+proc Scan_FolderUpdate { folder } {
     global ftoc
 
     if !$ftoc(displayValid) {
         return                  ;#  don't update pseudo-displays
     }
-    Label_Folder $f
-    Scan_Folder $f 0
+    Label_Folder $folder
+    Scan_Folder $folder 0
 }
 proc Scan_Iterate { incout lineVar body } {
     upvar $lineVar line
-    foreach line [split $incout \n] {
-	if [regexp ^Incorporating $line] {
+    foreach lineno [split $incout \n] {
+	if [regexp ^Incorporating $lineno] {
 	    continue
 	}
-	if {[string length $line] > 0} {
+	if {[string length $lineno] > 0} {
 	    uplevel $body
 	}
     }
@@ -154,17 +154,15 @@ proc Scan_Inc {folder incOutput} {
     global exwin ftoc
     # Append output of an Inc to the scan display
     ScanAddLineInit
-    Scan_Iterate $incOutput l {
-	ScanAddLine $l
+    Scan_Iterate $incOutput line {
+	ScanAddLine $line
     }
     ScanAddLineCleanup $folder
-    Ftoc_Update [Widget_TextEnd $exwin(ftext)] $folder
+    Ftoc_Update [Widget_TextEnd $exwin(ftext)]
     set ftoc(displayDirty) 1
     if {$ftoc(showNew)} {
 	Ftoc_Yview end
     }
-    # Don't forget unseen here, just find recently added unseen messages
-    Ftoc_ShowUnseen $folder
     Label_Folder $folder
 }
 proc Scan_IO {folder scanIO } {
@@ -178,7 +176,7 @@ proc ScanIO {folder scanIO } {
 	ScanAddLines [read $scanIO]
 	close $scanIO
     } err] {
-	Exmh_Status $err red
+	Exmh_Status $err warning
 	catch {close $scanIO}
     }
     ScanAddLineCleanup $folder
@@ -216,14 +214,13 @@ proc ScanAddLineCleanup { folder } {
     set ftoc(folder) $folder
     $exwin(ftext) configure -state disabled
 }
-proc Scan_ProjectSelection { ids } {
+proc Scan_ProjectSelection { msgids } {
     global ftoc exwin
     set lines {}
     set num 0
-    foreach id $ids {
-	set L [Ftoc_FindMsg $id]
-	if {$L != {}} {
-	    lappend lines [$exwin(ftext) get $L.0 $L.end]
+    foreach lineno [Ftoc_FindMsgs $msgids] {
+	if {$lineno != {}} {
+	    lappend lines [$exwin(ftext) get $lineno.0 $lineno.end]
 	    incr num
 	}
     }
@@ -233,17 +230,19 @@ proc Scan_ProjectSelection { ids } {
 	ScanAddLine $line
     }
     ScanAddLineCleanup $ftoc(folder)
+    Ftoc_ClearMsgCache
+    Ftoc_ShowSequences $msgids
     Msg_ClearCurrent
     Msg_Reset $num
 }
-proc Scan_CacheValid {F} {
+proc Scan_CacheValid {folder} {
     # Maintain a cache of folder listings
     global mhProfile exmh
-    set cacheFile $mhProfile(path)/$F/.xmhcache
+    set cacheFile $mhProfile(path)/$folder/.xmhcache
     if {![file exists $cacheFile] || ![file size $cacheFile]} {
 	return 0
     }
-    if {[file mtime $mhProfile(path)/$F] >
+    if {[file mtime $mhProfile(path)/$folder] >
 	[file mtime $cacheFile]} {
 	return 0
     }
@@ -274,12 +273,12 @@ proc Scan_CacheUpdate {} {
                 Exmh_Status "failed to rescan folder $folder: $err" warn
             }
         }
-	Ftoc_Change [Ftoc_MsgNumber $curLine] $curLine	;# Restore it
+	Ftoc_Change $curLine	;# Restore it
     } elseif [catch {
 	set cacheIO [open $cacheFile w]
 	set curLine [Ftoc_ClearCurrent]			;# Clear +
 	set display [$exwin(ftext) get 1.0 "end -1 char"]
-	Ftoc_Change [Ftoc_MsgNumber $curLine] $curLine	;# Restore it
+	Ftoc_Change $curLine	;# Restore it
 	puts $cacheIO $display nonewline
 	close $cacheIO
 	set ftoc(displayDirty) 0
@@ -343,6 +342,6 @@ proc Scan_AllFolders { {force 0} } {
     puts $out "exec rm $ctx.\[pid\]"
     puts $out exit
     close $out
-    Exmh_Status "wish -f [Env_Tmp]/scancmds &" blue
+    Exmh_Status "wish -f [Env_Tmp]/scancmds &"
     exec $wish -f [Env_Tmp]/scancmds &
 }
